@@ -48,7 +48,7 @@ void t_enum::append_value(std::unique_ptr<t_enum_value> enum_value) {
   const_val->set_enum_value(enum_value.get());
   auto tconst = std::make_unique<t_const>(
       program_,
-      &t_base_type::t_i32(),
+      &t_primitive_type::t_i32(),
       enum_value->get_name(),
       std::move(const_val));
   append(std::move(enum_value), std::move(tconst));
@@ -62,6 +62,14 @@ const t_enum_value* t_enum::find_value(int32_t value) const {
   return nullptr;
 }
 
+const t_const* t_enum::find_const_by_name(std::string_view name) const {
+  auto itr = consts_by_name_.find(name);
+  if (itr != consts_by_name_.end()) {
+    return itr->second;
+  }
+  return nullptr;
+}
+
 void t_enum::update_unused(int32_t val) {
   if (unused_ != val) {
     return;
