@@ -28,17 +28,14 @@ namespace compiler {
 class t_program;
 
 /**
- * An interaction is a service with context.
+ * Represents an interaction definition.
+ *
+ * An interaction is an interface with context.
  */
 // TODO(afuller): Inherit from t_interface directly.
 class t_interaction : public t_service {
  public:
   using t_service::t_service;
-
-  // TODO(afuller): Remove everything below this comment. It is only provided
-  // for backwards compatibility.
- public:
-  bool is_serial() const { return has_annotation("serial"); }
 };
 
 } // namespace compiler
