@@ -35,9 +35,7 @@ namespace compiler {
 class t_program;
 
 /**
- * A struct is a container for a set of member fields that has a name. Structs
- * are also used to implement exception and union types.
- *
+ * Represents a struct definition.
  */
 class t_struct : public t_structured {
  public:
@@ -48,7 +46,7 @@ class t_struct : public t_structured {
   // for backwards compatibility.
  public:
   bool is_struct() const override { return !is_exception(); }
-  type get_type_value() const override { return type::t_struct; }
+  type get_type_value() const override { return type::t_structured; }
 
   using t_structured::clone_DO_NOT_USE;
 
