@@ -16,10 +16,12 @@
 
 #pragma once
 
+#include <any>
 #include <array>
 #include <functional>
 #include <iosfwd>
 #include <string>
+#include <tuple>
 #include <utility>
 #include <vector>
 
@@ -38,6 +40,59 @@ enum class diagnostic_level {
   debug,
 };
 
+class fixit {
+ public:
+  /**
+   * @param original   - original text to be fixed
+   * @param replacement - replacement text to replace the original
+   * @param loc         - location of the fixit in the source code, if left
+   * blank will be populated with the location of the diagnostic
+   */
+  fixit(std::string original, std::string replacement, source_location loc = {})
+      : original_(std::move(original)),
+        replacement_(std::move(replacement)),
+        loc_(std::move(loc)) {}
+
+  fixit(std::string original, std::string replacement, int line, int column)
+      : original_(std::move(original)),
+        replacement_(std::move(replacement)),
+        line_(line),
+        column_(column) {}
+
+  const std::string& original() const { return original_; }
+  const std::string& replacement() const { return replacement_; }
+  int lineno() const { return line_; }
+  int column() const { return column_; }
+
+  // Resolve the location of the fixit in the source code, or use the provided
+  // diagonstic location
+  void resolve_location(
+      source_manager& sm, source_location diagnostic_location) {
+    source_location fixit_location =
+        loc_ != source_location() ? loc_ : diagnostic_location;
+    if (fixit_location != source_location()) {
+      auto resolved_loc = resolved_location(fixit_location, sm);
+      this->line_ = resolved_loc.line();
+      this->column_ = resolved_loc.column();
+    }
+  }
+
+ private:
+  std::string original_;
+  std::string replacement_;
+  int line_ = 0;
+  int column_ = 0;
+  source_location loc_;
+
+  // Comparing two fixit hints, not including the source_location since that
+  // is an optional param that ultimately just gets resolved into line and
+  // column
+  friend bool operator==(const fixit& lhs, const fixit& rhs) {
+    return std::tie(lhs.original_, lhs.replacement_, lhs.line_, lhs.column_) ==
+        std::tie(rhs.original_, rhs.replacement_, rhs.line_, rhs.column_);
+  }
+};
+
 /**
  * A diagnostic message.
  */
@@ -51,24 +106,28 @@ class diagnostic {
    * @param file       - file path location of diagnostic
    * @param line       - line location of diagnostic in the file, if known
    * @param name       - name given to this diagnostic, if any
+   * @param fixit      - an optional fix to apply to the source code
    */
   diagnostic(
       diagnostic_level level,
       std::string message,
       std::string file,
       int line = 0,
-      std::string name = "")
+      std::string name = "",
+      std::optional<fixit> fixit_hint = {})
       : level_(level),
         message_(std::move(message)),
         file_(std::move(file)),
         line_(line),
-        name_(std::move(name)) {}
+        name_(std::move(name)),
+        fixit_hint_(std::move(fixit_hint)) {}
 
   diagnostic_level level() const { return level_; }
   const std::string& message() const { return message_; }
   const std::string& file() const { return file_; }
   int lineno() const { return line_; }
   const std::string& name() const { return name_; }
+  const std::optional<fixit>& fixit_hint() const { return fixit_hint_; }
   std::string str() const;
 
   void set_name(std::string&& name) { name_ = std::move(name); }
@@ -79,11 +138,23 @@ class diagnostic {
   std::string file_;
   int line_;
   std::string name_;
+  std::optional<fixit> fixit_hint_;
 
   friend bool operator==(const diagnostic& lhs, const diagnostic& rhs) {
-    return lhs.level_ == rhs.level_ && lhs.line_ == rhs.line_ &&
-        lhs.message_ == rhs.message_ && lhs.file_ == rhs.file_ &&
-        lhs.name_ == rhs.name_;
+    return std::tie(
+               lhs.level_,
+               lhs.line_,
+               lhs.message_,
+               lhs.file_,
+               lhs.name_,
+               lhs.fixit_hint_) ==
+        std::tie(
+               rhs.level_,
+               rhs.line_,
+               rhs.message_,
+               rhs.file_,
+               rhs.name_,
+               rhs.fixit_hint_);
   }
   friend bool operator!=(const diagnostic& lhs, const diagnostic& rhs) {
     return !(lhs == rhs);
@@ -202,7 +273,28 @@ class diagnostics_engine {
       diagnostic_level level,
       fmt::format_string<T...> msg,
       T&&... args) {
-    do_report(loc.loc, {}, level, fmt::format(msg, std::forward<T>(args)...));
+    do_report(
+        loc.loc,
+        {}, /* name */
+        {}, /* fixit_hint */
+        level,
+        fmt::format(msg, std::forward<T>(args)...));
+  }
+
+  template <typename... T>
+  void report(
+      diagnostic_location loc,
+      std::string name,
+      std::optional<fixit> fixit_hint,
+      diagnostic_level level,
+      fmt::format_string<T...> msg,
+      T&&... args) {
+    do_report(
+        loc.loc,
+        std::move(name),
+        std::move(fixit_hint),
+        level,
+        fmt::format(msg, std::forward<T>(args)...));
   }
 
   template <typename... T>
@@ -215,6 +307,7 @@ class diagnostics_engine {
     do_report(
         loc.loc,
         std::move(name),
+        {}, /* fixit_hint */
         level,
         fmt::format(msg, std::forward<T>(args)...));
   }
@@ -256,6 +349,7 @@ class diagnostics_engine {
   void do_report(
       source_location loc,
       std::string name,
+      std::optional<fixit> fixit_hint,
       diagnostic_level level,
       std::string msg);
 
