@@ -29,6 +29,7 @@
 #include <thrift/compiler/detail/mustache/mstch.h>
 #include <thrift/compiler/lib/cpp2/util.h>
 #include <thrift/compiler/lib/uri.h>
+#include <thrift/compiler/sema/sema_context.h>
 
 namespace apache {
 namespace thrift {
@@ -235,6 +236,8 @@ struct mstch_context : mstch_factories {
   std::unordered_map<std::string, std::shared_ptr<mstch_base>> service_cache;
   std::unordered_map<std::string, std::shared_ptr<mstch_base>> program_cache;
 
+  node_metadata_cache metadata_cache;
+
   /**
    * Sets or erases the option with the given `key` depending on the
    * `condition`.
@@ -246,6 +249,8 @@ struct mstch_context : mstch_factories {
    */
   mstch_context& set_or_erase_option(
       bool condition, const std::string& key, const std::string& value);
+
+  node_metadata_cache& cache() { return metadata_cache; }
 };
 
 std::shared_ptr<mstch_base> make_mstch_program_cached(
@@ -328,12 +333,17 @@ class mstch_base : public mstch::object {
     return make_mstch_array(container, context_.service_factory.get(), args...);
   }
 
-  template <typename C>
+  template <typename C, typename... Args>
   mstch::array make_mstch_interactions(
-      const C& container, const t_service* containing_service) {
+      const C& container,
+      const t_service* containing_service,
+      const Args&... args) {
     if (context_.interaction_factory) {
       return make_mstch_array(
-          container, *context_.interaction_factory, containing_service);
+          container,
+          *context_.interaction_factory,
+          args...,
+          containing_service);
     }
     return make_mstch_array(container, *context_.service_factory);
   }
@@ -590,6 +600,7 @@ class mstch_service : public mstch_base {
     return service_->has_annotation("process_in_event_base") ||
         service_->find_structured_annotation_or_null(kCppProcessInEbThreadUri);
   }
+  mstch::node definition_key();
 
   virtual ~mstch_service() = default;
 
