@@ -147,6 +147,7 @@ class py3_mstch_program : public mstch_program {
             {"program:containerTypes", &py3_mstch_program::getContainerTypes},
             {"program:hasPyContainerTypes",
              &py3_mstch_program::hasPyContainerTypes},
+            {"program:hasEnumTypes", &py3_mstch_program::hasEnumTypes},
             {"program:customTemplates", &py3_mstch_program::getCustomTemplates},
             {"program:customTypes", &py3_mstch_program::getCustomTypes},
             {"program:moveContainerTypes",
@@ -196,6 +197,18 @@ class py3_mstch_program : public mstch_program {
     return false;
   }
 
+  mstch::node hasEnumTypes() {
+    if (!program_->enums().empty()) {
+      return true;
+    }
+    for (const auto* ttype : objects_) {
+      if (ttype->is_union()) {
+        return true;
+      }
+    }
+    return false;
+  }
+
   mstch::node getCppIncludes() {
     mstch::array a;
     if (program_->language_includes().count("cpp")) {
@@ -1124,7 +1137,7 @@ std::string py3_mstch_program::visit_type_with_typedef(
 // Generator-specific validator that enforces that a reserved key is not used
 // as a namespace component.
 void validate_no_reserved_key_in_namespace(
-    diagnostic_context& ctx, const t_program& prog) {
+    sema_context& ctx, const t_program& prog) {
   auto namespace_tokens = get_py3_namespace(&prog);
   if (namespace_tokens.empty()) {
     return;
@@ -1159,8 +1172,7 @@ void validate_no_reserved_key_in_namespace(
 // Generator-specific validator that enforces "name" and "value" are not used
 // as enum member or union field names (thrift-py3).
 namespace enum_member_union_field_names_validator {
-void validate(
-    const t_named& node, const std::string& name, diagnostic_context& ctx) {
+void validate(const t_named& node, const std::string& name, sema_context& ctx) {
   auto pyname = node.get_annotation("py3.name", &name);
   if (const t_const* annot =
           node.find_structured_annotation_or_null(kPythonNameUri)) {
@@ -1180,14 +1192,14 @@ void validate(
         pyname);
   }
 }
-bool validate_enum(diagnostic_context& ctx, const t_enum& enm) {
+bool validate_enum(sema_context& ctx, const t_enum& enm) {
   for (const t_enum_value& ev : enm.values()) {
     validate(ev, ev.get_name(), ctx);
   }
   return true;
 }
 
-bool validate_union(diagnostic_context& ctx, const t_union& s) {
+bool validate_union(sema_context& ctx, const t_union& s) {
   for (const t_field& f : s.fields()) {
     validate(f, f.name(), ctx);
   }
@@ -1389,6 +1401,7 @@ void t_mstch_py3_generator::generate_types() {
       "types_empty.pyx",
       "types_fields.pxd",
       "types_fields.pyx",
+      "types_impl_FBTHRIFT_ONLY_DO_NOT_USE.py",
       "types_reflection.py",
   };
 
