@@ -15,13 +15,14 @@
  */
 
 #include <algorithm>
+#include <filesystem>
 #include <memory>
 #include <set>
 #include <stdexcept>
 #include <string>
 
-#include <boost/algorithm/string/join.hpp>
 #include <boost/algorithm/string/replace.hpp>
+#include <fmt/format.h>
 
 #include <thrift/compiler/ast/t_program.h>
 #include <thrift/compiler/ast/t_type.h>
@@ -182,8 +183,9 @@ std::vector<std::string> gather_import_modules(
     }
 
     if (has_types(prog)) {
-      import_modules.push_back(boost::algorithm::join(
-          get_py_namespaces_raw(prog, is_asyncio, "ttypes"), "."));
+      import_modules.push_back(fmt::format(
+          "{}",
+          fmt::join(get_py_namespaces_raw(prog, is_asyncio, "ttypes"), ".")));
     }
   }
 
@@ -204,7 +206,7 @@ class pyi_mstch_program : public mstch_program {
       mstch_context& context,
       mstch_element_position position)
       : mstch_program(program, context, position) {
-    register_methods(
+    register_cached_methods(
         this,
         {
             {"program:returnTypes", &pyi_mstch_program::get_return_types},
@@ -361,7 +363,7 @@ class pyi_mstch_field : public mstch_field {
       mstch_element_position position,
       const field_generator_context* field_context)
       : mstch_field(field, context, position, field_context) {
-    register_methods(
+    register_cached_methods(
         this,
         {
             {"field:requireValue?", &pyi_mstch_field::get_require_value},
@@ -420,7 +422,7 @@ class pyi_mstch_type : public mstch_type {
       mstch_element_position position,
       const t_program* program)
       : mstch_type(type, context, position), program_(program) {
-    register_methods(
+    register_cached_methods(
         this,
         {
             {"type:modulePath", &pyi_mstch_type::get_module_path},
@@ -469,7 +471,7 @@ class pyi_mstch_service : public mstch_service {
       mstch_element_position position,
       const t_program* program)
       : mstch_service(service, context, position), program_(program) {
-    register_methods(
+    register_cached_methods(
         this,
         {
             {"service:externalProgram?",
@@ -533,7 +535,7 @@ class pyi_mstch_function : public mstch_function {
       mstch_element_position position,
       const t_interface* iface)
       : mstch_function(function, context, position, iface) {
-    register_methods(
+    register_cached_methods(
         this,
         {
             {"function:isSupported?", &pyi_mstch_function::is_supported},
@@ -559,7 +561,7 @@ class t_mstch_pyi_generator : public t_mstch_generator {
   void generate_program() override;
 
  private:
-  boost::filesystem::path root_path_;
+  std::filesystem::path root_path_;
 
   bool should_resolve_typedefs() const override { return true; }
 
@@ -570,9 +572,9 @@ class t_mstch_pyi_generator : public t_mstch_generator {
   void generate_services();
   void render_file(
       const std::string& template_name,
-      const boost::filesystem::path& path,
+      const std::filesystem::path& path,
       const t_service* service = nullptr);
-  boost::filesystem::path get_root_path() const;
+  std::filesystem::path get_root_path() const;
 };
 
 void t_mstch_pyi_generator::generate_program() {
@@ -596,7 +598,7 @@ void t_mstch_pyi_generator::create_factories() {
 }
 
 void t_mstch_pyi_generator::generate_init_files() {
-  boost::filesystem::path directory;
+  std::filesystem::path directory;
   for (const auto& part : this->root_path_) {
     directory /= part;
 
@@ -625,7 +627,7 @@ void t_mstch_pyi_generator::generate_services() {
 
 void t_mstch_pyi_generator::render_file(
     const std::string& template_name,
-    const boost::filesystem::path& path,
+    const std::filesystem::path& path,
     const t_service* service) {
   auto mstchObject = (service == nullptr)
       ? make_mstch_program_cached(
@@ -636,15 +638,15 @@ void t_mstch_pyi_generator::render_file(
   t_mstch_generator::render_to_file(mstchObject, template_name, path);
 }
 
-boost::filesystem::path t_mstch_pyi_generator::get_root_path() const {
-  boost::filesystem::path path;
+std::filesystem::path t_mstch_pyi_generator::get_root_path() const {
+  std::filesystem::path path;
 
   auto namespaces = get_py_namespaces_raw(
       this->get_program(), t_mstch_generator::has_option("asyncio"));
   for (const auto& ns : namespaces) {
     path /= ns;
   }
-  path += boost::filesystem::path::preferred_separator;
+  path += std::filesystem::path::preferred_separator;
 
   return path;
 }
