@@ -677,17 +677,16 @@ class t_mstch_python_capi_generator : public t_mstch_generator {
   }
 
   void fill_validator_visitors(ast_validator& validator) const override {
-    validator.add_field_visitor([](diagnostic_context& ctx, const t_field& f) {
+    validator.add_field_visitor([](sema_context& ctx, const t_field& f) {
       if (f.is_injected()) {
         return;
       }
       validate_explicit_include(ctx, f, *f.get_type(), diagnostic_level::error);
     });
-    validator.add_typedef_visitor(
-        [](diagnostic_context& ctx, const t_typedef& td) {
-          validate_explicit_include(
-              ctx, td, *td.get_type(), diagnostic_level::error);
-        });
+    validator.add_typedef_visitor([](sema_context& ctx, const t_typedef& td) {
+      validate_explicit_include(
+          ctx, td, *td.get_type(), diagnostic_level::error);
+    });
   }
 
  protected:
