@@ -503,7 +503,7 @@ class python_mstch_interaction : public python_mstch_service {
 // Generator-specific validator that enforces that a reserved key is not used
 // as a namespace component.
 void validate_no_reserved_key_in_namespace(
-    diagnostic_context& ctx, const t_program& prog) {
+    sema_context& ctx, const t_program& prog) {
   auto namespace_tokens = get_py3_namespace(&prog);
   if (namespace_tokens.empty()) {
     return;
@@ -886,8 +886,7 @@ class python_mstch_enum_value : public mstch_enum_value {
 // Generator-specific validator that enforces "name" and "value" are not used
 // as enum member or union field names (thrift-py3).
 namespace enum_member_union_field_names_validator {
-void validate(
-    const t_named* node, const std::string& name, diagnostic_context& ctx) {
+void validate(const t_named* node, const std::string& name, sema_context& ctx) {
   auto pyname = node->get_annotation("py3.name", &name);
   if (const t_const* annot =
           node->find_structured_annotation_or_null(kPythonNameUri)) {
@@ -907,7 +906,7 @@ void validate(
         pyname);
   }
 }
-bool validate_enum(diagnostic_context& ctx, const t_enum& enm) {
+bool validate_enum(sema_context& ctx, const t_enum& enm) {
   for (const t_enum_value* ev : enm.get_enum_values()) {
     validate(ev, ev->get_name(), ctx);
   }
@@ -915,7 +914,7 @@ bool validate_enum(diagnostic_context& ctx, const t_enum& enm) {
 }
 
 // TODO(T176314881): this never does anything?
-bool validate_structured(diagnostic_context& ctx, const t_struct& s) {
+bool validate_structured(sema_context& ctx, const t_struct& s) {
   if (!s.is_union()) {
     return false;
   }
@@ -931,6 +930,15 @@ class t_mstch_python_generator : public t_mstch_generator {
  public:
   using t_mstch_generator::t_mstch_generator;
 
+  std::optional<whisker_options> use_whisker() const override {
+    whisker_options opts;
+    opts.allowed_undefined_variables = {
+        "field:has_adapter?",
+        "field:type",
+    };
+    return opts;
+  }
+
   std::string template_prefix() const override { return "python"; }
 
   void generate_program() override {
