@@ -18,8 +18,6 @@
 
 #include <sstream>
 #include <string>
-#include <boost/algorithm/string/predicate.hpp>
-#include <boost/algorithm/string/replace.hpp>
 
 #include <thrift/compiler/ast/t_program.h>
 
