@@ -30,8 +30,6 @@ inline constexpr auto kExperimentalUri =
     "facebook.com/thrift/annotation/Experimental";
 inline constexpr auto kExceptionMessageUri =
     "facebook.com/thrift/annotation/ExceptionMessage";
-inline constexpr auto kGenerateRuntimeSchemaUri =
-    "facebook.com/thrift/annotation/GenerateRuntimeSchema";
 inline constexpr auto kInjectMetadataFieldsUri =
     "facebook.com/thrift/annotation/InjectMetadataFields";
 inline constexpr auto kMixinUri = "facebook.com/thrift/annotation/Mixin";
@@ -53,8 +51,10 @@ inline constexpr auto kUriUri = "facebook.com/thrift/annotation/Uri";
 inline constexpr auto kPriorityUri = "facebook.com/thrift/annotation/Priority";
 inline constexpr auto kDeprecatedUnvalidatedAnnotationsUri =
     "facebook.com/thrift/annotation/DeprecatedUnvalidatedAnnotations";
-inline constexpr auto kAllowReservedIdentifierNameUri =
-    "facebook.com/thrift/annotation/AllowReservedIdentifierName";
+inline constexpr auto kAllowReservedIdentifierUri =
+    "facebook.com/thrift/annotation/AllowReservedIdentifier";
+inline constexpr auto kAllowReservedFilenameUri =
+    "facebook.com/thrift/annotation/AllowReservedFilename";
 
 // scope
 inline constexpr auto kScopeProgramUri =
@@ -108,6 +108,8 @@ inline constexpr auto kCppTypeUri = "facebook.com/thrift/annotation/cpp/Type";
 inline constexpr auto kCppNameUri = "facebook.com/thrift/annotation/cpp/Name";
 inline constexpr auto kCppProcessInEbThreadUri =
     "facebook.com/thrift/annotation/cpp/ProcessInEbThreadUnsafe";
+inline constexpr auto kCppGenerateDeprecatedHeaderClientMethodsUri =
+    "facebook.com/thrift/annotation/cpp/GenerateDeprecatedHeaderClientMethods";
 
 // java
 inline constexpr auto kJavaMutableUri =
