@@ -44,7 +44,6 @@
 #include <thrift/compiler/lib/reserved_identifier.h>
 #include <thrift/compiler/lib/uri.h>
 #include <thrift/compiler/sema/explicit_include_validator.h>
-#include <thrift/compiler/sema/scope_validator.h>
 
 namespace apache {
 namespace thrift {
@@ -58,7 +57,7 @@ const t_structured* get_mixin_type(const t_field& field) {
   return nullptr;
 }
 
-bool has_experimental_annotation(diagnostic_context& ctx, const t_named& node) {
+bool has_experimental_annotation(sema_context& ctx, const t_named& node) {
   if (node.find_structured_annotation_or_null(kExperimentalUri)) {
     return true;
   }
@@ -169,7 +168,7 @@ class redef_checker {
 // Helper for validating the adapters
 class adapter_or_wrapper_checker {
  public:
-  explicit adapter_or_wrapper_checker(diagnostic_context& ctx) : ctx_(ctx) {}
+  explicit adapter_or_wrapper_checker(sema_context& ctx) : ctx_(ctx) {}
 
   // Checks if adapter name is provided
   // Do not allow composing two structured annotations on typedef
@@ -292,7 +291,7 @@ class adapter_or_wrapper_checker {
   }
 
  private:
-  diagnostic_context& ctx_;
+  sema_context& ctx_;
 };
 
 struct service_metadata {
@@ -332,30 +331,45 @@ struct structured_metadata {
 };
 
 void validate_identifier_is_not_reserved(
-    diagnostic_context& ctx, const t_named& node) {
+    sema_context& ctx, const t_named& node) {
   // Bypass the check any of the cases below.
   //  1. the node was generated.
-  //  2. @thrift.AllowedReservedIdentifierName is present.
+  //  2. @thrift.AllowedReservedIdentifier is present.
   if (node.generated() ||
-      node.find_structured_annotation_or_null(
-          kAllowReservedIdentifierNameUri) != nullptr) {
+      node.find_structured_annotation_or_null(kAllowReservedIdentifierUri) !=
+          nullptr) {
     return;
   }
   ctx.check(
       !is_reserved_identifier(node.name()),
-      "`{}` is a reserved identifier name. Choose a different name that does not contain `fbthrift`.",
+      "`{}` is a reserved identifier. Choose a different identifier that does not contain `fbthrift`.",
+      node.name());
+}
+
+void validate_filename_is_not_reserved(sema_context& ctx, const t_named& node) {
+  // Bypass the check any of the cases below.
+  //  1. the node was generated.
+  //  2. @thrift.AllowReservedFilename is present.
+  if (node.generated() ||
+      node.find_structured_annotation_or_null(kAllowReservedFilenameUri) !=
+          nullptr) {
+    return;
+  }
+  ctx.check(
+      !is_reserved_identifier(node.name()),
+      "`{}` is a reserved filename. Choose a different filename that does not contain `fbthrift`.",
       node.name());
 }
 
 void validate_interface_function_name_uniqueness(
-    diagnostic_context& ctx, const t_interface& node) {
+    sema_context& ctx, const t_interface& node) {
   // Check for a redefinition of a function in the same interface.
   redef_checker(ctx, "Function", node).check_all(node.functions());
 }
 
 // Checks for a redefinition of an inherited function.
 void validate_extends_service_function_name_uniqueness(
-    diagnostic_context& ctx, const t_service& node) {
+    sema_context& ctx, const t_service& node) {
   if (node.extends() == nullptr) {
     return;
   }
@@ -376,7 +390,7 @@ void validate_extends_service_function_name_uniqueness(
   }
 }
 
-void validate_throws_exceptions(diagnostic_context& ctx, const t_throws& node) {
+void validate_throws_exceptions(sema_context& ctx, const t_throws& node) {
   for (const auto& except : node.fields()) {
     auto except_type = except.type()->get_true_type();
     ctx.check(
@@ -390,7 +404,7 @@ void validate_throws_exceptions(diagnostic_context& ctx, const t_throws& node) {
 // Checks for a redefinition of a field in the same t_structured, including
 // those inherited via mixin fields.
 void validate_field_names_uniqueness(
-    diagnostic_context& ctx, const t_structured& node) {
+    sema_context& ctx, const t_structured& node) {
   redef_checker checker(ctx, "Field", node);
   for (const auto& field : node.fields()) {
     // Check the directly defined field.
@@ -410,7 +424,7 @@ void validate_field_names_uniqueness(
 // This validator checks if the node that contains any field
 // with that annotation is an exception definiton.
 void validate_exception_message_annotation_is_only_in_exceptions(
-    diagnostic_context& ctx, const t_structured& node) {
+    sema_context& ctx, const t_structured& node) {
   for (const auto& f : node.fields()) {
     if (f.find_structured_annotation_or_null(kExceptionMessageUri)) {
       ctx.check(
@@ -423,8 +437,7 @@ void validate_exception_message_annotation_is_only_in_exceptions(
 }
 
 // Checks the attributes of fields in a union.
-void validate_union_field_attributes(
-    diagnostic_context& ctx, const t_union& node) {
+void validate_union_field_attributes(sema_context& ctx, const t_union& node) {
   for (const auto& field : node.fields()) {
     if (field.qualifier() == t_field_qualifier::optional ||
         field.qualifier() == t_field_qualifier::required) {
@@ -444,8 +457,7 @@ void validate_union_field_attributes(
   }
 }
 
-void validate_boxed_field_attributes(
-    diagnostic_context& ctx, const t_field& node) {
+void validate_boxed_field_attributes(sema_context& ctx, const t_field& node) {
   if (gen::cpp::find_ref_type(node) == gen::cpp::reference_type::none) {
     return;
   }
@@ -504,8 +516,7 @@ void validate_boxed_field_attributes(
 }
 
 // Checks the attributes of a mixin field.
-void validate_mixin_field_attributes(
-    diagnostic_context& ctx, const t_field& node) {
+void validate_mixin_field_attributes(sema_context& ctx, const t_field& node) {
   if (!cpp2::is_mixin(node)) {
     return;
   }
@@ -531,7 +542,7 @@ void validate_mixin_field_attributes(
   }
 }
 
-void validate_required_field(diagnostic_context& ctx, const t_field& field) {
+void validate_required_field(sema_context& ctx, const t_field& field) {
   if (field.qualifier() == t_field_qualifier::required) {
     ctx.warning(
         field,
@@ -541,12 +552,11 @@ void validate_required_field(diagnostic_context& ctx, const t_field& field) {
 }
 
 void validate_enum_value_name_uniqueness(
-    diagnostic_context& ctx, const t_enum& node) {
+    sema_context& ctx, const t_enum& node) {
   redef_checker(ctx, "Enum value", node).check_all(node.values());
 }
 
-void validate_enum_value_uniqueness(
-    diagnostic_context& ctx, const t_enum& node) {
+void validate_enum_value_uniqueness(sema_context& ctx, const t_enum& node) {
   std::unordered_map<int32_t, const t_enum_value*> values;
   for (const auto& value : node.values()) {
     auto prev = values.emplace(value.get_value(), &value);
@@ -561,7 +571,7 @@ void validate_enum_value_uniqueness(
   }
 }
 
-void validate_enum_value(diagnostic_context& ctx, const t_enum_value& node) {
+void validate_enum_value(sema_context& ctx, const t_enum_value& node) {
   if (!node.has_value()) {
     ctx.error(
         "The enum value, `{}`, must have an explicitly assigned value.",
@@ -569,8 +579,7 @@ void validate_enum_value(diagnostic_context& ctx, const t_enum_value& node) {
   }
 }
 
-void validate_const_type_and_value(
-    diagnostic_context& ctx, const t_const& node) {
+void validate_const_type_and_value(sema_context& ctx, const t_const& node) {
   detail::check_initializer(ctx, node, node.type(), node.value());
   ctx.check(
       !node.find_structured_annotation_or_null(kCppAdapterUri) ||
@@ -579,15 +588,14 @@ void validate_const_type_and_value(
       node.name());
 }
 
-void validate_field_default_value(
-    diagnostic_context& ctx, const t_field& field) {
+void validate_field_default_value(sema_context& ctx, const t_field& field) {
   if (field.get_default_value() != nullptr) {
     detail::check_initializer(
         ctx, field, &field.type().deref(), field.default_value());
   }
 }
 
-void validate_field_name(diagnostic_context& ctx, const t_field& field) {
+void validate_field_name(sema_context& ctx, const t_field& field) {
   const auto* strct = dynamic_cast<const t_structured*>(ctx.parent());
   if (field.get_name() == strct->get_name()) {
     std::string parent_structure;
@@ -605,8 +613,7 @@ void validate_field_name(diagnostic_context& ctx, const t_field& field) {
   }
 }
 
-void validate_structured_annotation(
-    diagnostic_context& ctx, const t_named& node) {
+void validate_structured_annotation(sema_context& ctx, const t_named& node) {
   std::unordered_map<const t_type*, const t_const*> seen;
   for (const t_const& annot : node.structured_annotations()) {
     auto [it, inserted] = seen.emplace(annot.type(), &annot);
@@ -623,7 +630,7 @@ void validate_structured_annotation(
   }
 }
 
-void validate_uri_uniqueness(diagnostic_context& ctx, const t_program& prog) {
+void validate_uri_uniqueness(sema_context& ctx, const t_program& prog) {
   // TODO: use string_view as map key
   std::unordered_map<std::string, const t_named*> uri_to_node;
   basic_ast_visitor<true> visit;
@@ -645,7 +652,7 @@ void validate_uri_uniqueness(diagnostic_context& ctx, const t_program& prog) {
 }
 
 void limit_terse_write_on_experimental_mode(
-    diagnostic_context& ctx, const t_named& node) {
+    sema_context& ctx, const t_named& node) {
   ctx.check(
       !node.find_structured_annotation_or_null(kTerseWriteUri) ||
           has_experimental_annotation(ctx, node),
@@ -653,7 +660,7 @@ void limit_terse_write_on_experimental_mode(
       node.name());
 }
 
-void validate_field_id(diagnostic_context& ctx, const t_field& node) {
+void validate_field_id(sema_context& ctx, const t_field& node) {
   if (node.explicit_id() != node.id()) {
     ctx.warning(
         node,
@@ -676,7 +683,7 @@ void validate_field_id(diagnostic_context& ctx, const t_field& node) {
 }
 
 void validate_compatibility_with_lazy_field(
-    diagnostic_context& ctx, const t_structured& node) {
+    sema_context& ctx, const t_structured& node) {
   if (has_lazy_field(node) && node.has_annotation("cpp.methods")) {
     ctx.error(
         "cpp.methods is incompatible with lazy deserialization in struct `{}`",
@@ -684,7 +691,7 @@ void validate_compatibility_with_lazy_field(
   }
 }
 
-void validate_ref_annotation(diagnostic_context& ctx, const t_field& node) {
+void validate_ref_annotation(sema_context& ctx, const t_field& node) {
   if (node.find_structured_annotation_or_null(kCppRefUri) &&
       node.has_annotation(
           {"cpp.ref", "cpp2.ref", "cpp.ref_type", "cpp2.ref_type"})) {
@@ -695,50 +702,45 @@ void validate_ref_annotation(diagnostic_context& ctx, const t_field& node) {
   }
 }
 
-void validate_cpp_adapter_annotation(
-    diagnostic_context& ctx, const t_named& node) {
+void validate_cpp_adapter_annotation(sema_context& ctx, const t_named& node) {
   adapter_or_wrapper_checker(ctx).check(
       node, kCppAdapterUri, "@cpp.Adapter", "name");
 }
 
-void validate_hack_adapter_annotation(
-    diagnostic_context& ctx, const t_named& node) {
+void validate_hack_adapter_annotation(sema_context& ctx, const t_named& node) {
   adapter_or_wrapper_checker(ctx).check(
       node, kHackAdapterUri, "@hack.Adapter", "name");
 }
 
-void validate_hack_wrapper_annotation(
-    diagnostic_context& ctx, const t_named& node) {
+void validate_hack_wrapper_annotation(sema_context& ctx, const t_named& node) {
   adapter_or_wrapper_checker(ctx).check(
       node, kHackWrapperUri, "@hack.Wrapper", "name");
 }
 // Do not adapt a wrapped type
 void validate_hack_wrapper_and_adapter_annotation(
-    diagnostic_context& ctx, const t_named& node) {
+    sema_context& ctx, const t_named& node) {
   adapter_or_wrapper_checker(ctx).check(
       node, kHackAdapterUri, kHackWrapperUri, "@hack.Adapter", "@hack.Wrapper");
 }
 
-void validate_java_adapter_annotation(
-    diagnostic_context& ctx, const t_named& node) {
+void validate_java_adapter_annotation(sema_context& ctx, const t_named& node) {
   adapter_or_wrapper_checker(ctx).check(
       node, kJavaAdapterUri, "@java.Adapter", "adapterClassName");
 }
 
-void validate_java_wrapper_annotation(
-    diagnostic_context& ctx, const t_named& node) {
+void validate_java_wrapper_annotation(sema_context& ctx, const t_named& node) {
   adapter_or_wrapper_checker(ctx).check(
       node, kJavaWrapperUri, "@java.Wrapper", "wrapperClassName");
 }
 
 void validate_java_wrapper_and_adapter_annotation(
-    diagnostic_context& ctx, const t_named& node) {
+    sema_context& ctx, const t_named& node) {
   adapter_or_wrapper_checker(ctx).check(
       node, kJavaAdapterUri, kJavaWrapperUri, "@java.Adapter", "@java.Wrapper");
 }
 
 void validate_ref_unique_and_box_annotation(
-    diagnostic_context& ctx, const t_field& node) {
+    sema_context& ctx, const t_field& node) {
   const t_const* adapter_annotation =
       node.find_structured_annotation_or_null(kCppAdapterUri);
 
@@ -787,7 +789,7 @@ void validate_ref_unique_and_box_annotation(
 }
 
 void validate_function_priority_annotation(
-    diagnostic_context& ctx, const t_node& node) {
+    sema_context& ctx, const t_node& node) {
   if (auto* priority = node.find_annotation_or_null("priority")) {
     const std::string choices[] = {
         "HIGH_IMPORTANT", "HIGH", "IMPORTANT", "NORMAL", "BEST_EFFORT"};
@@ -801,7 +803,7 @@ void validate_function_priority_annotation(
 }
 
 void validate_exception_message_annotation(
-    diagnostic_context& ctx, const t_exception& node) {
+    sema_context& ctx, const t_exception& node) {
   // Check that value of "message" annotation is
   // - a valid member of struct
   // - of type STRING
@@ -840,7 +842,7 @@ void validate_exception_message_annotation(
 }
 
 void validate_interaction_nesting(
-    diagnostic_context& ctx, const t_interaction& node) {
+    sema_context& ctx, const t_interaction& node) {
   for (auto* func : node.get_functions()) {
     if (func->interaction()) {
       ctx.error(*func, "Nested interactions are forbidden: {}", func->name());
@@ -849,7 +851,7 @@ void validate_interaction_nesting(
 }
 
 void validate_interaction_annotations(
-    diagnostic_context& ctx, const t_interaction& node) {
+    sema_context& ctx, const t_interaction& node) {
   for (auto* func : node.get_functions()) {
     ctx.check(
         !func->has_annotation("thread") &&
@@ -867,7 +869,7 @@ void validate_interaction_annotations(
 }
 
 void validate_cpp_field_interceptor_annotation(
-    diagnostic_context& ctx, const t_field& field) {
+    sema_context& ctx, const t_field& field) {
   if (const t_const* annot =
           field.find_structured_annotation_or_null(kCppFieldInterceptorUri)) {
     try {
@@ -881,7 +883,7 @@ void validate_cpp_field_interceptor_annotation(
   }
 }
 
-void validate_cpp_enum_type(diagnostic_context& ctx, const t_enum& e) {
+void validate_cpp_enum_type(sema_context& ctx, const t_enum& e) {
   if (const t_const* annot =
           e.find_structured_annotation_or_null(kCppEnumTypeUri)) {
     try {
@@ -896,7 +898,7 @@ void validate_cpp_enum_type(diagnostic_context& ctx, const t_enum& e) {
 }
 
 void validate_cpp_field_adapter_annotation(
-    diagnostic_context& ctx, const t_field& field) {
+    sema_context& ctx, const t_field& field) {
   adapter_or_wrapper_checker(ctx).check(
       field,
       kCppAdapterUri,
@@ -905,7 +907,7 @@ void validate_cpp_field_adapter_annotation(
 }
 
 void validate_hack_field_adapter_annotation(
-    diagnostic_context& ctx, const t_field& field) {
+    sema_context& ctx, const t_field& field) {
   adapter_or_wrapper_checker(ctx).check(
       field,
       kHackAdapterUri,
@@ -914,7 +916,7 @@ void validate_hack_field_adapter_annotation(
 }
 
 void validate_java_field_adapter_annotation(
-    diagnostic_context& ctx, const t_field& field) {
+    sema_context& ctx, const t_field& field) {
   adapter_or_wrapper_checker(ctx).check(
       field,
       kJavaAdapterUri,
@@ -924,7 +926,7 @@ void validate_java_field_adapter_annotation(
 
 class reserved_ids_checker {
  public:
-  explicit reserved_ids_checker(diagnostic_context& ctx) : ctx_(ctx) {}
+  explicit reserved_ids_checker(sema_context& ctx) : ctx_(ctx) {}
 
   void check(const t_structured& node) {
     auto reserved_ids = get_reserved_ids(node);
@@ -950,7 +952,7 @@ class reserved_ids_checker {
   }
 
  private:
-  diagnostic_context& ctx_;
+  sema_context& ctx_;
 
   // Gets all the reserved ids annotated on this node. Returns
   // empty set if the annotation is not present.
@@ -1032,11 +1034,11 @@ class reserved_ids_checker {
 };
 
 void validate_reserved_ids_structured(
-    diagnostic_context& ctx, const t_structured& node) {
+    sema_context& ctx, const t_structured& node) {
   reserved_ids_checker(ctx).check(node);
 }
 
-void validate_reserved_ids_enum(diagnostic_context& ctx, const t_enum& node) {
+void validate_reserved_ids_enum(sema_context& ctx, const t_enum& node) {
   reserved_ids_checker(ctx).check(node);
 }
 
@@ -1060,7 +1062,7 @@ bool owns_annotations(t_type_ref type) {
 }
 
 void validate_custom_cpp_type_annotations(
-    diagnostic_context& ctx, const t_named& node) {
+    sema_context& ctx, const t_named& node) {
   const bool hasAdapter =
       node.find_structured_annotation_or_null(kCppAdapterUri);
   bool hasCppType = node.has_annotation(
@@ -1114,7 +1116,7 @@ void validate_custom_cpp_type_annotations(
 }
 
 template <typename Node>
-void validate_cpp_type_annotation(diagnostic_context& ctx, const Node& node) {
+void validate_cpp_type_annotation(sema_context& ctx, const Node& node) {
   if (const t_const* annot =
           node.find_structured_annotation_or_null(kCppTypeUri)) {
     auto type = annot->get_value_from_structured_annotation_or_null("name");
@@ -1136,12 +1138,12 @@ void validate_cpp_type_annotation(diagnostic_context& ctx, const Node& node) {
 }
 
 struct ValidateAnnotationPositions {
-  void operator()(diagnostic_context& ctx, const t_const& node) {
+  void operator()(sema_context& ctx, const t_const& node) {
     if (owns_annotations(node.type())) {
       err(ctx);
     }
   }
-  void operator()(diagnostic_context& ctx, const t_function& node) {
+  void operator()(sema_context& ctx, const t_function& node) {
     if (owns_annotations(node.return_type())) {
       err(ctx);
     }
@@ -1164,7 +1166,7 @@ struct ValidateAnnotationPositions {
       }
     }
   }
-  void operator()(diagnostic_context& ctx, const t_container& type) {
+  void operator()(sema_context& ctx, const t_container& type) {
     switch (type.get_type_value()) {
       case t_type::type::t_list:
         if (owns_annotations(static_cast<const t_list&>(type).elem_type())) {
@@ -1189,13 +1191,13 @@ struct ValidateAnnotationPositions {
   }
 
  private:
-  static void err(diagnostic_context& ctx) {
+  static void err(sema_context& ctx) {
     ctx.error(
         "Annotations are not allowed in this position. Extract the type into a named typedef instead.");
   }
 };
 
-void deprecate_annotations(diagnostic_context& ctx, const t_named& node) {
+void deprecate_annotations(sema_context& ctx, const t_named& node) {
   auto erlang = [](std::string_view name) {
     return fmt::format("facebook.com/thrift/annotation/erlang/{}", name);
   };
@@ -1279,14 +1281,14 @@ bool has_cursor_serialization_adapter(const Node& node) {
 }
 
 void validate_cursor_serialization_adapter_on_field(
-    diagnostic_context& ctx, const t_field& node) {
+    sema_context& ctx, const t_field& node) {
   ctx.check(
       !has_cursor_serialization_adapter(node),
       "CursorSerializationAdapter is not supported on fields. Place it on the top-level struct/union instead.");
 }
 
 void validate_cursor_serialization_adapter_on_function(
-    diagnostic_context& ctx, const t_function& node) {
+    sema_context& ctx, const t_function& node) {
   if (node.params().fields().size() <= 1) {
     return;
   }
@@ -1299,7 +1301,7 @@ void validate_cursor_serialization_adapter_on_function(
 }
 
 void validate_cursor_serialization_adapter_in_container(
-    diagnostic_context& ctx, const t_container& node) {
+    sema_context& ctx, const t_container& node) {
   auto check = [&](const t_type& type) {
     ctx.check(
         !has_cursor_serialization_adapter(type),
@@ -1317,7 +1319,7 @@ void validate_cursor_serialization_adapter_in_container(
 
 // TODO (T191018859): forbid as field type too
 void forbid_exception_as_method_type(
-    diagnostic_context& ctx, const t_function& node) {
+    sema_context& ctx, const t_function& node) {
   ctx.check(
       !node.return_type()->get_true_type()->is_exception(),
       "Exceptions cannot be used as function return types");
@@ -1327,8 +1329,7 @@ void forbid_exception_as_method_type(
         "Exceptions cannot be used as function arguments");
   }
 }
-void forbid_exception_as_const_type(
-    diagnostic_context& ctx, const t_const& node) {
+void forbid_exception_as_const_type(sema_context& ctx, const t_const& node) {
   ctx.check(
       !node.type()->get_true_type()->is_exception(),
       "Exceptions cannot be used as const types");
@@ -1338,7 +1339,8 @@ void forbid_exception_as_const_type(
 
 ast_validator standard_validator() {
   ast_validator validator;
-  validator.add_named_visitor(&validate_identifier_is_not_reserved);
+  validator.add_definition_visitor(&validate_identifier_is_not_reserved);
+  validator.add_program_visitor(&validate_filename_is_not_reserved);
 
   validator.add_interface_visitor(&validate_interface_function_name_uniqueness);
   validator.add_interface_visitor(&validate_function_priority_annotation);
@@ -1380,7 +1382,7 @@ ast_validator standard_validator() {
   validator.add_enum_value_visitor(&validate_enum_value);
 
   validator.add_named_visitor(&validate_structured_annotation);
-  validator.add_named_visitor(&validate_annotation_scopes);
+  validator.add_named_visitor(&detail::validate_annotation_scopes);
   validator.add_named_visitor(&validate_cpp_adapter_annotation);
   validator.add_named_visitor(&validate_hack_adapter_annotation);
   validator.add_named_visitor(&validate_hack_wrapper_annotation);
