@@ -17,27 +17,14 @@ import json
 import os
 import shutil
 import subprocess
-import sys
 import tempfile
 import textwrap
 import unittest
 
-
-def ascend_find_exe(path, target):
-    if not os.path.isdir(path):
-        path = os.path.dirname(path)
-    while True:
-        test = os.path.join(path, target)
-        if os.access(test, os.X_OK):
-            return test
-        parent = os.path.dirname(path)
-        if os.path.samefile(parent, path):
-            return None
-        path = parent
+import pkg_resources
 
 
-exe = os.path.join(os.getcwd(), sys.argv[0])
-thrift = ascend_find_exe(exe, "thrift")
+thrift = pkg_resources.resource_filename(__name__, "thrift")
 
 
 def read_file(path):
@@ -177,7 +164,7 @@ class JsonCompilerTest(unittest.TestCase):
                                     },
                                     "end": {
                                         "line": 4,
-                                        "column": 21,
+                                        "column": 22,
                                     },
                                 },
                             },
@@ -257,7 +244,7 @@ class JsonCompilerTest(unittest.TestCase):
                                     },
                                     "end": {
                                         "line": 4,
-                                        "column": 34,
+                                        "column": 35,
                                     },
                                 },
                             },
@@ -330,7 +317,7 @@ class JsonCompilerTest(unittest.TestCase):
                                     },
                                     "end": {
                                         "line": 2,
-                                        "column": 21,
+                                        "column": 22,
                                     },
                                 },
                             },
@@ -389,7 +376,7 @@ class JsonCompilerTest(unittest.TestCase):
                                     },
                                     "end": {
                                         "line": 2,
-                                        "column": 17,
+                                        "column": 18,
                                     },
                                 },
                             },
