@@ -21,6 +21,7 @@
 #include <cstdint>
 #include <functional>
 #include <iosfwd>
+#include <limits>
 #include <map>
 #include <memory>
 #include <optional>
@@ -31,6 +32,7 @@
 #include <vector>
 
 #include <thrift/compiler/whisker/detail/overload.h>
+#include <thrift/compiler/whisker/tree_printer.h>
 
 namespace whisker {
 
@@ -72,6 +74,37 @@ class object;
 using map = std::map<std::string, object, std::less<>>;
 using array = std::vector<object>;
 
+/**
+ * Options for whisker::to_string() and whisker::print_to().
+ */
+struct object_print_options {
+  /**
+   * The maximum recursive depth of the object to print before truncating the
+   * output. This is useful for:
+   *   - Preventing infinite recursion when printing a cyclic object
+   *   - Preventing excessive output when printing a deeply nested object
+   *
+   * The depth value refers to the depth of the whisker::object hierarchy, which
+   * may not necessarily be the same as the depth of the printed tree.
+   *
+   * When the depth is reached, the output may be truncated based on the type of
+   * object being printed:
+   *   - For whisker::map, the property names are printed but values are
+   *     truncated.
+   *   - For whisker::array, the array indices are printed by elements are
+   *     truncated.
+   *   - For whisker::native_object, the behavior is implementation-defined. See
+   *     whisker::native_object::print_to().
+   *
+   * For all other whisker::object types, no truncation occurs because they do
+   * not incur additional depth.
+   *
+   * Truncated output appears as "...". This indicates that there are values
+   * beyond the specified depth that remain unexpanded.
+   */
+  unsigned max_depth = std::numeric_limits<unsigned>::max();
+};
+
 /**
  * A native_object is the most powerful type in Whisker. Its properties and
  * behavior are fully defined by user C++ code.
@@ -108,6 +141,74 @@ class native_object {
    *   - The provided string is a valid Whisker identifier
    */
   virtual const object* lookup_property(std::string_view identifier) const = 0;
+
+  /**
+   * Creates a tree-like string representation of this object, primarily for
+   * debugging and diagnostic purposes.
+   *
+   * The provided tree_printer::scope object abstracts away the details of tree
+   * printing as well as encodes the location in an existing print tree where
+   * this object should be inline. See its documentation for more details.
+   */
+  virtual void print_to(tree_printer::scope, const object_print_options&) const;
+
+  /**
+   * Determines if this native_object compares equal to the other object. It is
+   * the implementers's responsibility to ensure that the other object also
+   * compares equal to this one and maintains the commutative property.
+   *
+   * The default implementation compares by object identity.
+   */
+  virtual bool operator==(const native_object& other) const;
+
+  /**
+   * A class that allows "array-like" random access over an underlying sequence
+   * of objects.
+   */
+  class sequence {
+   public:
+    virtual ~sequence() = default;
+    /**
+     * Returns the number of elements in the sequence.
+     */
+    virtual std::size_t size() const = 0;
+    /**
+     * Returns the object at the specified index within the sequence.
+     *
+     * Preconditions:
+     *   - index < size()
+     */
+    virtual const object& at(std::size_t index) const = 0;
+  };
+  /**
+   * Returns an implementation of sequence if this object supports array-like
+   * iteration. Otherwise, returns nullptr.
+   *
+   * When this function returns a non-null pointer, the returned object can be
+   * used in Whisker template looping constructs, such as section blocks.
+   *
+   * This function returns a shared_ptr so that the returned object can be this
+   * object itself (which is expected to be stored as a native_object::ptr).
+   * Doing so prevents an extra heap allocation in favor of an atomic increment.
+   *
+   *     class my_object : public native_object,
+   *                       public native_object::sequence,
+   *                       public std::enable_shared_from_this<my_object> {
+   *      public:
+   *       std::shared_ptr<const sequence> as_sequence() const override {
+   *         return shared_from_this();
+   *       }
+   *
+   *       // Other functions...
+   *     };
+   *
+   * An array-like object is still allowed to have named properties supported
+   * via lookup_property(). This function opts in to strictly additional
+   * capabilities.
+   */
+  virtual std::shared_ptr<const sequence> as_sequence() const {
+    return nullptr;
+  }
 };
 
 namespace detail {
@@ -281,7 +382,8 @@ class object final : private detail::object_base<object> {
 
   friend bool operator==(
       const object& lhs, const native_object::ptr& rhs) noexcept {
-    return lhs.is_native_object() && lhs.as_native_object() == rhs;
+    return lhs.is_native_object() && rhs != nullptr &&
+        *lhs.as_native_object() == *rhs;
   }
   friend bool operator==(
       const native_object::ptr& lhs, const object& rhs) noexcept {
@@ -384,7 +486,15 @@ class object final : private detail::object_base<object> {
   }
 };
 
-std::string to_string(const object&);
+/**
+ * An alternative to to_string() that allows printing a whisker::object within
+ * an ongoing tree printing session. The output is appended to the provided
+ * tree_printer::scope's output stream.
+ *
+ * This is primarily needed for native_object::print_to() implementations.
+ */
+void print_to(const object&, tree_printer::scope, const object_print_options&);
+std::string to_string(const object&, const object_print_options& = {});
 
 std::ostream& operator<<(std::ostream&, const object&);
 
