@@ -16,7 +16,7 @@
 
 #pragma once
 
-#include <folly/experimental/coro/Task.h>
+#include <folly/coro/Task.h>
 #include <folly/stats/Histogram.h>
 #include <thrift/conformance/stresstest/if/gen-cpp2/StressTest.h>
 
@@ -51,6 +51,7 @@ class StressTestClient {
   virtual folly::coro::Task<void> co_requestResponseTm(const BasicRequest&) = 0;
   virtual folly::coro::Task<void> co_streamTm(const StreamRequest&) = 0;
   virtual folly::coro::Task<void> co_sinkTm(const StreamRequest&) = 0;
+  virtual folly::coro::Task<double> co_calculateSquares(int32_t) = 0;
 
   bool connectionGood() const { return connectionGood_; }
 
@@ -82,6 +83,8 @@ class ThriftStressTestClient : public StressTestClient {
 
   folly::coro::Task<void> co_sinkTm(const StreamRequest&) override;
 
+  folly::coro::Task<double> co_calculateSquares(int32_t) override;
+
  private:
   template <class Fn>
   folly::coro::Task<void> timedExecute(Fn&& fn);
