@@ -22,7 +22,9 @@
 
 #include <thrift/conformance/stresstest/util/IoUringUtil.h>
 
+#include <rroeser/scripts/src/executor/WorkStealingExecutor.h>
 #include <folly/executors/CPUThreadPoolExecutor.h>
+#include <thrift/lib/cpp2/server/ParallelConcurrencyController.h>
 #include <thrift/lib/cpp2/server/ThriftServer.h>
 
 DEFINE_int32(port, 5000, "Server port");
@@ -33,7 +35,15 @@ DEFINE_int32(
     -1,
     "Configures max requests, 0 will disable max request limit");
 DEFINE_bool(io_uring, false, "Enables io_uring if available when set to true");
-DEFINE_bool(no_resource_pile_or_cc, false, "Disable resource pile and cc");
+DEFINE_bool(
+    work_stealing,
+    false,
+    "Enable work stealing test. Implies --enable_resource_pools");
+DEFINE_bool(
+    work_stealing_executor_only,
+    false,
+    "Enable work stealing test without request pile or concurrency controller."
+    " Implies --work_stealing and --enable_resource_pools");
 DEFINE_string(
     certPath,
     "folly/io/async/test/certs/tests-cert.pem",
@@ -46,12 +56,28 @@ DEFINE_string(
     caPath,
     "folly/io/async/test/certs/ca-cert.pem",
     "Path to client trusted CA file");
+DEFINE_bool(enable_overload_checker, false, "Enable overload checker");
+DEFINE_bool(enable_resource_pools, false, "Enable resource pools");
+DEFINE_bool(
+    disable_active_request_tracking, false, "Disabled Active Request Tracking");
 
 namespace apache {
 namespace thrift {
 namespace stress {
 
 namespace {
+
+bool enableResourcePools() {
+  // These flags explicitly or implicitly enable resource pools.
+  return FLAGS_enable_resource_pools || FLAGS_work_stealing ||
+      FLAGS_work_stealing_executor_only;
+}
+
+bool enableWorkStealing() {
+  // These flags explicitly or implicitly enable work stealing executor.
+  return FLAGS_work_stealing || FLAGS_work_stealing_executor_only;
+}
+
 uint32_t sanitizeNumThreads(int32_t n) {
   return n <= 0 ? std::thread::hardware_concurrency() : n;
 }
@@ -121,6 +147,12 @@ std::shared_ptr<ThriftServer> createStressTestServer(
       getIOThreadPool("thrift_eventbase", FLAGS_io_threads));
   server->setNumCPUWorkerThreads(sanitizeNumThreads(FLAGS_cpu_threads));
 
+  LOG(INFO) << "Active Request Tracking Disabled: "
+            << FLAGS_disable_active_request_tracking;
+  if (FLAGS_disable_active_request_tracking) {
+    server->disableActiveRequestsTracking();
+  }
+
   if (FLAGS_max_requests > -1) {
     LOG(INFO) << "Setting max server requests: " << FLAGS_max_requests;
     server->setMaxRequests(FLAGS_max_requests);
@@ -130,15 +162,36 @@ std::shared_ptr<ThriftServer> createStressTestServer(
       !FLAGS_caPath.empty()) {
     server->setSSLConfig(getSSLConfig());
   }
-  if (FLAGS_no_resource_pile_or_cc) {
-    server->resourcePoolSet().setResourcePool(
-        apache::thrift::ResourcePoolHandle::defaultAsync(),
-        std::make_shared<folly::CPUThreadPoolExecutor>(
-            sanitizeNumThreads(FLAGS_cpu_threads)));
 
+  if (enableResourcePools()) {
+    LOG(INFO) << "Resource pools enabled";
+    std::shared_ptr<folly::Executor> executor;
+    std::unique_ptr<RequestPileInterface> requestPile;
+    std::unique_ptr<ConcurrencyControllerInterface> concurrencyController;
+    if (enableWorkStealing()) {
+      LOG(INFO) << "Work stealing executor enabled";
+      auto t = sanitizeNumThreads(FLAGS_cpu_threads);
+      executor = std::make_shared<folly::WorkStealingExecutor>(t, 4 * t);
+      if (FLAGS_work_stealing_executor_only) {
+        LOG(INFO) << "Request pile and concurrency controller disabled";
+      } else {
+        RoundRobinRequestPile::Options options;
+        requestPile =
+            std::make_unique<RoundRobinRequestPile>(std::move(options));
+        concurrencyController = std::make_unique<ParallelConcurrencyController>(
+            *requestPile.get(), *executor.get());
+      }
+    }
+
+    server->resourcePoolSet().setResourcePool(
+        ResourcePoolHandle::defaultAsync(),
+        std::move(requestPile),
+        executor,
+        std::move(concurrencyController));
     server->ensureResourcePools();
     server->requireResourcePools();
   }
+
   return server;
 }
 
