@@ -38,9 +38,9 @@ using is_mutable_ref = folly::Conjunction<
     std::is_reference<T>,
     folly::Negation<std::is_const<std::remove_reference_t<T>>>>;
 
-// The type returned by the adapter for the given thrift type.
+// The type returned by the adapter for the given Thrift type.
 template <typename Adapter, typename ThriftT>
-using adapted_t = decltype(Adapter::fromThrift(std::declval<ThriftT&&>()));
+using adapted_t = decltype(Adapter::fromThrift(std::declval<ThriftT>()));
 
 // Used to detect if Adapter has the fromThriftField function which takes an
 // additional FieldContext argument.
@@ -133,7 +133,7 @@ constexpr if_not_field_adapter<Adapter, ThriftT, Struct> fromThriftField(
 // The type returned by the adapter for the given thrift type of a struct field.
 template <typename Adapter, int16_t FieldId, typename ThriftT, typename Struct>
 using adapted_field_t = decltype(fromThriftField<Adapter, FieldId>(
-    std::declval<ThriftT&&>(), std::declval<Struct&>()));
+    std::declval<ThriftT>(), std::declval<Struct&>()));
 
 // The type returned by the adapter for the given adapted type.
 template <typename Adapter, typename AdaptedT>
