@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,14 +24,8 @@
 #include <thrift/lib/cpp/concurrency/Util.h>
 #include <thrift/lib/cpp/transport/TTransportException.h>
 
-using folly::IOBuf;
-using folly::IOBufQueue;
-using std::pair;
-using std::unique_ptr;
-using namespace folly::io;
 using namespace apache::thrift::transport;
 using folly::EventBase;
-using namespace apache::thrift::concurrency;
 
 namespace apache {
 namespace thrift {
