@@ -202,6 +202,10 @@ folly::Try<FirstResponsePayload> decodeResponseError(
       exCode = kTenantQuotaExceededErrorCode;
       exType = TApplicationException::LOADSHEDDING;
       break;
+    case ResponseRpcErrorCode::TENANT_BLOCKLISTED:
+      exCode = kTenantBlocklistedErrorCode;
+      exType = TApplicationException::TENANT_BLOCKLISTED;
+      break;
     default:
       exCode = kUnknownErrorCode;
       break;
@@ -736,15 +740,24 @@ rocket::SetupFrame RocketClientChannel::makeSetupFrame(
   if (!clientMetadata.agent_ref()) {
     clientMetadata.agent_ref() = "RocketClientChannel.cpp";
   }
-  CompactProtocolWriter compactProtocolWriter;
+
   folly::IOBufQueue paramQueue;
-  compactProtocolWriter.setOutput(&paramQueue);
-  meta.write(&compactProtocolWriter);
+  uint32_t serialized_size;
+  if (THRIFT_FLAG(rocket_client_binary_rpc_metadata_encoding)) {
+    BinaryProtocolWriter binaryProtocolWriter;
+    binaryProtocolWriter.setOutput(&paramQueue);
+    meta.write(&binaryProtocolWriter);
+    serialized_size = meta.serializedSize(&binaryProtocolWriter);
+  } else {
+    CompactProtocolWriter compactProtocolWriter;
+    compactProtocolWriter.setOutput(&paramQueue);
+    meta.write(&compactProtocolWriter);
+    serialized_size = meta.serializedSize(&compactProtocolWriter);
+  }
 
   // Serialize RocketClient's major/minor version (which is separate from the
   // rsocket protocol major/minor version) into setup metadata.
-  auto buf = folly::IOBuf::createCombined(
-      sizeof(int32_t) + meta.serializedSize(&compactProtocolWriter));
+  auto buf = folly::IOBuf::createCombined(sizeof(int32_t) + serialized_size);
   folly::IOBufQueue queue;
   queue.append(std::move(buf));
   folly::io::QueueAppender appender(&queue, /* do not grow */ 0);
@@ -760,7 +773,7 @@ rocket::SetupFrame RocketClientChannel::makeSetupFrame(
 
   return rocket::SetupFrame(
       rocket::Payload::makeFromMetadataAndData(queue.move(), {}),
-      /* rocketMimeTypes = */ true);
+      THRIFT_FLAG(rocket_client_binary_rpc_metadata_encoding));
 }
 
 RocketClientChannel::RocketClientChannel(
