@@ -16,7 +16,7 @@
 
 #include <thrift/lib/cpp2/async/ClientBufferedStream.h>
 
-#include <folly/experimental/coro/Baton.h>
+#include <folly/coro/Baton.h>
 #include <folly/io/async/ScopedEventBaseThread.h>
 #include <folly/portability/GTest.h>
 #include <thrift/lib/cpp2/async/ServerStream.h>
