@@ -17,6 +17,7 @@
 #pragma once
 
 #include <stdexcept>
+#include <folly/CPortability.h>
 #include <folly/Overload.h>
 #include <folly/Traits.h>
 #include <thrift/lib/cpp/Field.h>
@@ -127,6 +128,9 @@ using get_field_tag = typename std::conditional_t<
 template <typename T, typename Id>
 using get_native_type = type::native_type<get_field_tag<T, Id>>;
 
+FOLLY_PUSH_WARNING
+FOLLY_CLANG_DISABLE_WARNING("-Wglobal-constructors")
+
 /// Gets the thrift field name, for example:
 ///
 /// * op::get_name_v<MyStruct, field_id<7>>
@@ -144,6 +148,8 @@ template <typename T>
 inline const folly::StringPiece get_class_name_v =
     detail::pa::__fbthrift_get_class_name<T>();
 
+FOLLY_POP_WARNING
+
 /// Gets the Thrift field, for example:
 ///
 ///   op::get<type::field_id<7>>(myStruct) = 4;
