@@ -30,6 +30,9 @@
 
 namespace apache {
 namespace thrift {
+namespace ident {
+struct patch;
+}
 namespace op {
 
 class bad_patch_access : public std::runtime_error {
@@ -44,7 +47,8 @@ namespace detail {
 /// - Patch: The Thrift struct representation for the patch.
 /// - Derived: The leaf type deriving from this class.
 template <typename Patch, typename Derived>
-class BasePatch : public type::detail::EqWrap<Derived, Patch> {
+class BasePatch
+    : public type::detail::EqWrap<Derived, Patch, type::struct_t<Patch>> {
   using Base = type::detail::EqWrap<Derived, Patch>;
 
  public:
@@ -106,19 +110,19 @@ class BasePatch : public type::detail::EqWrap<Derived, Patch> {
       std::forward<U>(next).customVisit(derived());
     }
   }
-  [[deprecated("Use customVisit(...) method to read the patch.")]] auto&&
+  [[deprecated("ERROR: Use customVisit(...) method to read the patch.")]] auto&&
   toThrift() & {
     return Base::toThrift();
   }
-  [[deprecated("Use customVisit(...) method to read the patch.")]] auto&&
+  [[deprecated("ERROR: Use customVisit(...) method to read the patch.")]] auto&&
   toThrift() && {
     return Base::toThrift();
   }
-  [[deprecated("Use customVisit(...) method to read the patch.")]] auto&&
+  [[deprecated("ERROR: Use customVisit(...) method to read the patch.")]] auto&&
   toThrift() const& {
     return Base::toThrift();
   }
-  [[deprecated("Use customVisit(...) method to read the patch.")]] auto&&
+  [[deprecated("ERROR: Use customVisit(...) method to read the patch.")]] auto&&
   toThrift() const&& {
     return Base::toThrift();
   }
