@@ -55,20 +55,6 @@ void clear_impl(MaskRef ref, T& obj, Id id, Value& value) {
   ref.clear(value);
 }
 
-// call copy based on the type of the value.
-protocol::Value filter(MaskRef ref, const Value& src) {
-  protocol::Value ret;
-  if (src.is_object()) {
-    ret.emplace_object(ref.filter(src.as_object()));
-  } else if (src.is_map()) {
-    ret.emplace_map(ref.filter(src.as_map()));
-  } else {
-    folly::throw_exception<std::runtime_error>(
-        "The mask and object are incompatible.");
-  }
-  return ret;
-}
-
 template <typename T, typename Id>
 void filter_impl(MaskRef ref, const T& src, T& ret, Id id) {
   // Id doesn't exist in field mask, skip.
@@ -190,7 +176,7 @@ MaskRef MaskRef::get(const std::string& key) const {
 }
 
 MaskRef MaskRef::get(const type::Type& type) const {
-  if (isAllMask() || isNoneMask()) { // This whole map is included or excluded.
+  if (isAllMask() || isNoneMask()) { // All types included or excluded.
     return *this;
   }
   throwIfNotTypeMask();
@@ -203,6 +189,36 @@ MaskRef MaskRef::get(const type::Type& type) const {
   }
 }
 
+// Uses type::identicalType to look up the nested MaskRef for the given type
+// Use this if type can possibly be a hashed URI
+MaskRef MaskRef::getViaIdenticalType_INTERNAL_DO_NOT_USE(
+    const type::Type& type) const {
+  if (type.isFull()) {
+    // Optimize for the common case of a full type.
+    return get(type);
+  }
+  if (isAllMask() || isNoneMask()) { // All types included or excluded.
+    return *this;
+  }
+  throwIfNotTypeMask();
+  auto findViaIdenticalType = [&](const auto& map) -> const Mask& {
+    for (auto it = map.begin(); it != map.end(); ++it) {
+      if (type::identicalType(it->first, type)) {
+        return it->second;
+      }
+    }
+    return field_mask_constants::noneMask();
+  };
+
+  if (mask.includes_type_ref()) {
+    return MaskRef{
+        findViaIdenticalType(mask.includes_type_ref().value()), is_exclusion};
+  } else {
+    return MaskRef{
+        findViaIdenticalType(mask.excludes_type_ref().value()), !is_exclusion};
+  }
+}
+
 bool MaskRef::isAllMask() const {
   return (is_exclusion &&
           ::apache::thrift::protocol::detail::isNoneMask(mask)) ||
@@ -286,8 +302,8 @@ void MaskRef::clear(protocol::Object& obj) const {
           "Incompatible mask and data: expected Any for type-mask");
     };
 
-    // get mask for the type in any
-    auto nestedMask = get(any.type());
+    // The type of Any could be a hashed-URI so use getViaIdenticalType
+    auto nestedMask = getViaIdenticalType_INTERNAL_DO_NOT_USE(any.type());
     if (!nestedMask.isNoneMask()) {
       // recurse
       auto value = detail::parseValueFromAny(any);
@@ -317,7 +333,48 @@ void MaskRef::clear(folly::F14FastMap<Value, Value>& map) const {
   }
 }
 
+protocol::Value MaskRef::filter(const Value& src) const {
+  protocol::Value ret;
+  if (isAllMask()) {
+    ret = src;
+  } else if (src.is_object()) {
+    ret.emplace_object(filter(src.as_object()));
+  } else if (src.is_map()) {
+    ret.emplace_map(filter(src.as_map()));
+  } else {
+    folly::throw_exception<std::runtime_error>(
+        "The mask and object are incompatible.");
+  }
+  return ret;
+}
+
 protocol::Object MaskRef::filter(const protocol::Object& src) const {
+  if (isTypeMask()) {
+    // obj -> any
+    type::AnyData any;
+    if (!detail::ProtocolValueToThriftValue<type::infer_tag<type::AnyData>>{}(
+            src, any)) {
+      folly::throw_exception<std::runtime_error>(
+          "Incompatible mask and data: expected Any for type-mask");
+    };
+
+    // The type of Any could be a hashed-URI so use getViaIdenticalType
+    auto nestedMask = getViaIdenticalType_INTERNAL_DO_NOT_USE(any.type());
+    if (nestedMask.isNoneMask()) {
+      // Filter failed, return empty object
+      return {};
+    } else {
+      // Recurse
+      any = detail::toAny(
+          nestedMask.filter(detail::parseValueFromAny(any)),
+          any.type(),
+          any.protocol());
+    }
+
+    return detail::asValueStruct<type::infer_tag<type::AnyData>>(std::move(any))
+        .as_object();
+  }
+
   throwIfNotFieldMask();
   protocol::Object ret;
   for (auto& [fieldId, _] : src) {
