@@ -600,9 +600,7 @@ void Cpp2Worker::dispatchRequest(
         }
       } else if (
           // wildcard metadata do not specify rpcKind
-          (found->metadata.rpcKind ||
-           (found->metadata.isWildcard() &&
-            THRIFT_FLAG(allow_wildcard_process_via_execute_request))) &&
+          (found->metadata.rpcKind || found->metadata.isWildcard()) &&
           // executorType is defaulted to UNKNOWN for wildcard metadata
           // so only processors that implement createMethodMetadata can
           // pass this test
