@@ -31,15 +31,15 @@
 #include <folly/Memory.h>
 #include <folly/ScopeGuard.h>
 #include <folly/String.h>
+#include <folly/coro/BlockingWait.h>
+#include <folly/coro/Collect.h>
+#include <folly/coro/CurrentExecutor.h>
+#include <folly/coro/Invoke.h>
 #include <folly/executors/CPUThreadPoolExecutor.h>
 #include <folly/executors/IOThreadPoolDeadlockDetectorObserver.h>
 #include <folly/executors/thread_factory/InitThreadFactory.h>
 #include <folly/executors/thread_factory/NamedThreadFactory.h>
 #include <folly/executors/thread_factory/PriorityThreadFactory.h>
-#include <folly/experimental/coro/BlockingWait.h>
-#include <folly/experimental/coro/Collect.h>
-#include <folly/experimental/coro/CurrentExecutor.h>
-#include <folly/experimental/coro/Invoke.h>
 #include <folly/io/GlobalShutdownSocketSet.h>
 #include <folly/portability/Sockets.h>
 #include <folly/system/Pid.h>
@@ -50,6 +50,7 @@
 #include <thrift/lib/cpp/server/TServerObserver.h>
 #include <thrift/lib/cpp2/Flags.h>
 #include <thrift/lib/cpp2/async/MultiplexAsyncProcessor.h>
+#include <thrift/lib/cpp2/runtime/Init.h>
 #include <thrift/lib/cpp2/server/Cpp2Connection.h>
 #include <thrift/lib/cpp2/server/Cpp2Worker.h>
 #include <thrift/lib/cpp2/server/ExecutorToThreadManagerAdaptor.h>
@@ -330,6 +331,11 @@ void ThriftServer::initializeDefaults() {
           THRIFT_SERVER_EVENT(ACC_enabled).log(*this);
         }
       });
+
+  for (const auto& initializer :
+       apache::thrift::runtime::getGlobalServerInitializers()) {
+    initializer(*this);
+  }
 }
 
 std::unique_ptr<RequestPileInterface> ThriftServer::makeStandardRequestPile(
@@ -945,38 +951,42 @@ void ThriftServer::setupThreadManager() {
   // Now do setup that we want to do whether we created these resources or the
   // client did.
   if (!resourcePoolSet().empty()) {
-    auto cc = resourcePoolSet()
-                  .resourcePool(ResourcePoolHandle::defaultAsync())
-                  .concurrencyController();
-    if (cc.has_value()) {
-      // Keep concurrency controller in sync with max requests for now.
-      setMaxRequestsCallbackHandle =
-          detail::getThriftServerConfig(*this)
-              .getMaxRequests()
-              .getObserver()
-              .addCallback([cc = cc.value()](
-                               folly::observer::Snapshot<uint32_t> snapshot) {
-                auto maxRequests = *snapshot;
-                cc.get().setExecutionLimitRequests(
+    // Keep concurrency controller in sync with max requests for now.
+    setMaxRequestsCallbackHandle =
+        detail::getThriftServerConfig(*this)
+            .getMaxRequests()
+            .getObserver()
+            .addCallback([this](folly::observer::Snapshot<uint32_t> snapshot) {
+              auto maxRequests = *snapshot;
+              if (auto cc =
+                      resourcePoolSet()
+                          .resourcePool(ResourcePoolHandle::defaultAsync())
+                          .concurrencyController()) {
+                cc.value().get().setExecutionLimitRequests(
                     maxRequests != 0
                         ? maxRequests
                         : std::numeric_limits<decltype(maxRequests)>::max());
-              });
-      setMaxQpsCallbackHandle =
-          detail::getThriftServerConfig(*this)
-              .getMaxQps()
-              .getObserver()
-              .addCallback([cc = cc.value()](
-                               folly::observer::Snapshot<uint32_t> snapshot) {
-                auto maxQps = *snapshot;
-                cc.get().setQpsLimit(
+              }
+            });
+    setMaxQpsCallbackHandle =
+        detail::getThriftServerConfig(*this)
+            .getMaxQps()
+            .getObserver()
+            .addCallback([this](folly::observer::Snapshot<uint32_t> snapshot) {
+              auto maxQps = *snapshot;
+              if (auto cc =
+                      resourcePoolSet()
+                          .resourcePool(ResourcePoolHandle::defaultAsync())
+                          .concurrencyController()) {
+                cc.value().get().setQpsLimit(
                     maxQps != 0 ? maxQps
                                 : std::numeric_limits<decltype(maxQps)>::max());
-              });
-    }
+              }
+            });
     // Create an adapter so calls to getThreadManager_deprecated will work
     // when we are using resource pools
-    if (!threadManager_) {
+    if (!threadManager_ &&
+        resourcePoolSet().hasResourcePool(ResourcePoolHandle::defaultAsync())) {
       auto extm =
           std::make_shared<ExecutorToThreadManagerAdaptor>(resourcePoolSet());
       setThreadManagerInternal(extm);
@@ -1193,7 +1203,11 @@ void ThriftServer::ensureResourcePools() {
 
   if (!resourcePoolSet().hasResourcePool(ResourcePoolHandle::defaultSync())) {
     // Ensure there is a sync resource pool.
-    resourcePoolSet().setResourcePool(ResourcePoolHandle::defaultSync());
+    resourcePoolSet().setResourcePool(
+        ResourcePoolHandle::defaultSync(),
+        /*requestPile=*/nullptr,
+        /*executor=*/nullptr,
+        /*concurrencyController=*/nullptr);
   }
 
   // The user provided the resource pool. Use it as is.
