@@ -17,12 +17,12 @@
 #include <chrono>
 #include <memory>
 
-#include <folly/experimental/coro/BlockingWait.h>
-#include <folly/experimental/coro/Collect.h>
-#include <folly/experimental/coro/FutureUtil.h>
-#include <folly/experimental/coro/GtestHelpers.h>
-#include <folly/experimental/coro/Sleep.h>
-#include <folly/experimental/coro/Task.h>
+#include <folly/coro/BlockingWait.h>
+#include <folly/coro/Collect.h>
+#include <folly/coro/FutureUtil.h>
+#include <folly/coro/GtestHelpers.h>
+#include <folly/coro/Sleep.h>
+#include <folly/coro/Task.h>
 #include <folly/portability/GMock.h>
 #include <folly/portability/GTest.h>
 #include <thrift/lib/cpp2/async/RocketClientChannel.h>
@@ -883,7 +883,9 @@ TEST(InteractionCodegenTest, ReuseIdDuringConstructor) {
   {
     auto id = client.getChannel()->registerInteraction("Addition", 1);
     CalculatorAsyncClient::Addition adder(
-        client.getChannelShared(), std::move(id));
+        client.getChannelShared(),
+        std::move(id),
+        nullptr /* clientInterceptors */);
     adder.semifuture_noop().via(&eb).getVia(&eb);
     handler->b1.wait();
   } // sends termination while constructor is blocked
@@ -891,7 +893,9 @@ TEST(InteractionCodegenTest, ReuseIdDuringConstructor) {
 
   auto id = client.getChannel()->registerInteraction("Addition", 1);
   CalculatorAsyncClient::Addition adder(
-      client.getChannelShared(), std::move(id));
+      client.getChannelShared(),
+      std::move(id),
+      nullptr /* clientInterceptors */);
 
   auto fut = adder.semifuture_accumulatePrimitive(1);
   handler->b2.post();
