@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -74,8 +74,6 @@ class CoreTestFixture : public testing::Test {
   std::shared_ptr<testing::StrictMock<testutil::testservice::TestServiceMock>>
       service_ = std::make_shared<
           testing::StrictMock<testutil::testservice::TestServiceMock>>();
-  std::shared_ptr<concurrency::ThreadManager> threadManager_ =
-      concurrency::ThreadManager::newSimpleThreadManager(1);
   ThriftProcessor processor_;
   folly::EventBase eventBase_;
   std::shared_ptr<FakeChannel> channel_;
