@@ -42,16 +42,20 @@ class SetupFrame {
  public:
   explicit SetupFrame(std::unique_ptr<folly::IOBuf> frame);
 
-  explicit SetupFrame(Payload&& payload, bool rocketMimeTypes)
-      : payload_(std::move(payload)), rocketMimeTypes_(rocketMimeTypes) {}
+  explicit SetupFrame(Payload&& payload, bool encodeMetadataUsingBinary)
+      : payload_(std::move(payload)),
+        rocketMimeTypes_(true),
+        encodeMetadataUsingBinary_(encodeMetadataUsingBinary) {}
 
   static constexpr FrameType frameType() { return FrameType::SETUP; }
 
   size_t frameHeaderSize() const {
+    size_t metadataMimeTypeSize = encodeMetadataUsingBinary_
+        ? kRocketMetadataBinaryMimeType.size()
+        : kRocketMetadataCompactMimeType.size();
     size_t frameSize = 20 +
-        (rocketMimeTypes_
-             ? kRocketMetadataMimeType.size() + kRocketPayloadMimeType.size()
-             : 2 * kLegacyMimeType.size());
+        (rocketMimeTypes_ ? metadataMimeTypeSize + kRocketPayloadMimeType.size()
+                          : 2 * kLegacyMimeType.size());
     if (hasResumeIdentificationToken()) {
       frameSize +=
           2 /* bytes for token length */ + resumeIdentificationToken_.size();
@@ -70,14 +74,18 @@ class SetupFrame {
 
   bool rocketMimeTypes() const { return rocketMimeTypes_; }
 
+  bool encodeMetadataUsingBinary() const { return encodeMetadataUsingBinary_; }
+
   std::unique_ptr<folly::IOBuf> serialize() &&;
   void serialize(Serializer& writer) &&;
 
  private:
-  static constexpr folly::StringPiece kLegacyMimeType{"text/plain"};
-  static constexpr folly::StringPiece kRocketMetadataMimeType{
+  static constexpr std::string_view kLegacyMimeType{"text/plain"};
+  static constexpr std::string_view kRocketMetadataCompactMimeType{
       "application/x-rocket-metadata+compact"};
-  static constexpr folly::StringPiece kRocketPayloadMimeType{
+  static constexpr std::string_view kRocketMetadataBinaryMimeType{
+      "application/x-rocket-metadata+binary"};
+  static constexpr std::string_view kRocketPayloadMimeType{
       "application/x-rocket-payload"};
 
   // Resume ID token and Lease flags are not currently supported/used.
@@ -85,6 +93,7 @@ class SetupFrame {
   std::string resumeIdentificationToken_;
   Payload payload_;
   bool rocketMimeTypes_;
+  bool encodeMetadataUsingBinary_;
 };
 
 class RequestResponseFrame {
