@@ -31,7 +31,6 @@
 #include <thrift/lib/cpp2/Flags.h>
 #include <thrift/lib/cpp2/async/AsyncProcessorHelper.h>
 #include <thrift/lib/cpp2/async/ResponseChannel.h>
-#include <thrift/lib/cpp2/protocol/CompactProtocol.h>
 #include <thrift/lib/cpp2/server/Cpp2ConnContext.h>
 #include <thrift/lib/cpp2/server/Cpp2Worker.h>
 #include <thrift/lib/cpp2/server/LoggingEvent.h>
@@ -211,7 +210,7 @@ void ThriftRocketServerHandler::handleSetupFrame(
       reader.setInput(cursor);
       // Throws on read error
       meta.read(&reader);
-      if (reader.getCursorPosition() > frame.payload().metadataSize()) {
+      if (reader.getCursorPosition() != frame.payload().metadataSize()) {
         return connection.close(folly::make_exception_wrapper<RocketException>(
             ErrorCode::INVALID_SETUP,
             "Error deserializing SETUP payload: underflow"));
@@ -221,7 +220,7 @@ void ThriftRocketServerHandler::handleSetupFrame(
       reader.setInput(cursor);
       // Throws on read error
       meta.read(&reader);
-      if (reader.getCursorPosition() > frame.payload().metadataSize()) {
+      if (reader.getCursorPosition() != frame.payload().metadataSize()) {
         return connection.close(folly::make_exception_wrapper<RocketException>(
             ErrorCode::INVALID_SETUP,
             "Error deserializing SETUP payload: underflow"));
