@@ -86,6 +86,32 @@ struct TestStruct {
   void __fbthrift_clear() {}
 };
 
+template <typename T, typename Struct, int16_t FieldId>
+struct FieldValueWithContext {
+  T value;
+  std::string* meta = nullptr;
+};
+
+struct FieldAdapterWithContext {
+  template <typename T, typename Struct, int16_t FieldId>
+  static FieldValueWithContext<T, Struct, FieldId> fromThriftField(
+      T&& value, FieldContext<Struct, FieldId>&& context) {
+    return {std::forward<T>(value), &context.object.meta};
+  }
+
+  template <typename T, typename Struct, int16_t FieldId>
+  static const T& toThrift(
+      const FieldValueWithContext<T, Struct, FieldId>& value) {
+    return value.value;
+  }
+};
+
+struct TestStructWithContext {
+  void __fbthrift_clear() { meta.clear(); }
+  // custom default
+  std::string meta = "meta";
+};
+
 // Creates a custom protocol, skipping validation.
 inline type::Protocol makeProtocol(std::string name) {
   type::ProtocolUnion data;
