@@ -23,6 +23,8 @@ import (
 	"math"
 	"strconv"
 	"testing"
+
+	"github.com/facebook/fbthrift/thrift/lib/go/thrift/types"
 )
 
 func TestWriteJSONProtocolBool(t *testing.T) {
@@ -279,16 +281,16 @@ func TestWriteJSONProtocolDouble(t *testing.T) {
 		}
 		s := trans.String()
 		if math.IsInf(value, 1) {
-			if s != jsonQuote(JSON_INFINITY) {
-				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(JSON_INFINITY))
+			if s != jsonQuote(types.JSON_INFINITY) {
+				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(types.JSON_INFINITY))
 			}
 		} else if math.IsInf(value, -1) {
-			if s != jsonQuote(JSON_NEGATIVE_INFINITY) {
-				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(JSON_NEGATIVE_INFINITY))
+			if s != jsonQuote(types.JSON_NEGATIVE_INFINITY) {
+				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(types.JSON_NEGATIVE_INFINITY))
 			}
 		} else if math.IsNaN(value) {
-			if s != jsonQuote(JSON_NAN) {
-				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(JSON_NAN))
+			if s != jsonQuote(types.JSON_NAN) {
+				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(types.JSON_NAN))
 			}
 		} else {
 			if s != fmt.Sprint(value) {
@@ -309,7 +311,7 @@ func TestReadJSONProtocolDouble(t *testing.T) {
 	for _, value := range doubleValues {
 		trans := NewMemoryBuffer()
 		p := NewJSONProtocol(trans)
-		n := NewNumericFromDouble(value)
+		n := types.NewNumericFromDouble(value)
 		trans.WriteString(n.String())
 		s := trans.String()
 		v, e := p.ReadDouble()
@@ -354,16 +356,16 @@ func TestWriteJSONProtocolFloat(t *testing.T) {
 		}
 		s := trans.String()
 		if math.IsInf(float64(value), 1) {
-			if s != jsonQuote(JSON_INFINITY) {
-				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(JSON_INFINITY))
+			if s != jsonQuote(types.JSON_INFINITY) {
+				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(types.JSON_INFINITY))
 			}
 		} else if math.IsInf(float64(value), -1) {
-			if s != jsonQuote(JSON_NEGATIVE_INFINITY) {
-				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(JSON_NEGATIVE_INFINITY))
+			if s != jsonQuote(types.JSON_NEGATIVE_INFINITY) {
+				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(types.JSON_NEGATIVE_INFINITY))
 			}
 		} else if math.IsNaN(float64(value)) {
-			if s != jsonQuote(JSON_NAN) {
-				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(JSON_NAN))
+			if s != jsonQuote(types.JSON_NAN) {
+				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(types.JSON_NAN))
 			}
 		} else {
 			if s != fmt.Sprint(value) {
@@ -384,7 +386,7 @@ func TestReadJSONProtocolFloat(t *testing.T) {
 	for _, value := range floatValues {
 		trans := NewMemoryBuffer()
 		p := NewJSONProtocol(trans)
-		n := NewNumericFromFloat(value)
+		n := types.NewNumericFromFloat(value)
 		trans.WriteString(n.String())
 		s := trans.String()
 		v, e := p.ReadFloat()
@@ -531,7 +533,7 @@ func TestWriteJSONProtocolList(t *testing.T) {
 	thetype := "list"
 	trans := NewMemoryBuffer()
 	p := NewJSONProtocol(trans)
-	p.WriteListBegin(Type(DOUBLE), len(doubleValues))
+	p.WriteListBegin(types.Type(types.DOUBLE), len(doubleValues))
 	for _, value := range doubleValues {
 		if e := p.WriteDouble(value); e != nil {
 			t.Fatalf("Unable to write %s value %v due to error: %s", thetype, value, e.Error())
@@ -552,7 +554,7 @@ func TestWriteJSONProtocolList(t *testing.T) {
 		t.Fatalf("List must be at least of length two to include metadata")
 	}
 	if l[0] != "dbl" {
-		t.Fatal("Invalid type for list, expected: ", STRING, ", but was: ", l[0])
+		t.Fatal("Invalid type for list, expected: ", types.STRING, ", but was: ", l[0])
 	}
 	if int(l[1].(float64)) != len(doubleValues) {
 		t.Fatal("Invalid length for list, expected: ", len(doubleValues), ", but was: ", l[1])
@@ -560,16 +562,16 @@ func TestWriteJSONProtocolList(t *testing.T) {
 	for k, value := range doubleValues {
 		s := l[k+2]
 		if math.IsInf(value, 1) {
-			if s.(string) != JSON_INFINITY {
-				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(JSON_INFINITY), str)
+			if s.(string) != types.JSON_INFINITY {
+				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(types.JSON_INFINITY), str)
 			}
 		} else if math.IsInf(value, 0) {
-			if s.(string) != JSON_NEGATIVE_INFINITY {
-				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(JSON_NEGATIVE_INFINITY), str)
+			if s.(string) != types.JSON_NEGATIVE_INFINITY {
+				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(types.JSON_NEGATIVE_INFINITY), str)
 			}
 		} else if math.IsNaN(value) {
-			if s.(string) != JSON_NAN {
-				t.Fatalf("Bad value for %s at index %v  %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(JSON_NAN), str)
+			if s.(string) != types.JSON_NAN {
+				t.Fatalf("Bad value for %s at index %v  %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(types.JSON_NAN), str)
 			}
 		} else {
 			if s.(float64) != value {
@@ -585,7 +587,7 @@ func TestWriteJSONProtocolSet(t *testing.T) {
 	thetype := "set"
 	trans := NewMemoryBuffer()
 	p := NewJSONProtocol(trans)
-	p.WriteSetBegin(Type(DOUBLE), len(doubleValues))
+	p.WriteSetBegin(types.Type(types.DOUBLE), len(doubleValues))
 	for _, value := range doubleValues {
 		if e := p.WriteDouble(value); e != nil {
 			t.Fatalf("Unable to write %s value %v due to error: %s", thetype, value, e.Error())
@@ -606,7 +608,7 @@ func TestWriteJSONProtocolSet(t *testing.T) {
 		t.Fatalf("Set must be at least of length two to include metadata")
 	}
 	if l[0] != "dbl" {
-		t.Fatal("Invalid type for set, expected: ", DOUBLE, ", but was: ", l[0])
+		t.Fatal("Invalid type for set, expected: ", types.DOUBLE, ", but was: ", l[0])
 	}
 	if int(l[1].(float64)) != len(doubleValues) {
 		t.Fatal("Invalid length for set, expected: ", len(doubleValues), ", but was: ", l[1])
@@ -614,16 +616,16 @@ func TestWriteJSONProtocolSet(t *testing.T) {
 	for k, value := range doubleValues {
 		s := l[k+2]
 		if math.IsInf(value, 1) {
-			if s.(string) != JSON_INFINITY {
-				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(JSON_INFINITY), str)
+			if s.(string) != types.JSON_INFINITY {
+				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(types.JSON_INFINITY), str)
 			}
 		} else if math.IsInf(value, 0) {
-			if s.(string) != JSON_NEGATIVE_INFINITY {
-				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(JSON_NEGATIVE_INFINITY), str)
+			if s.(string) != types.JSON_NEGATIVE_INFINITY {
+				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(types.JSON_NEGATIVE_INFINITY), str)
 			}
 		} else if math.IsNaN(value) {
-			if s.(string) != JSON_NAN {
-				t.Fatalf("Bad value for %s at index %v  %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(JSON_NAN), str)
+			if s.(string) != types.JSON_NAN {
+				t.Fatalf("Bad value for %s at index %v  %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(types.JSON_NAN), str)
 			}
 		} else {
 			if s.(float64) != value {
@@ -639,7 +641,7 @@ func TestWriteJSONProtocolMap(t *testing.T) {
 	thetype := "map"
 	trans := NewMemoryBuffer()
 	p := NewJSONProtocol(trans)
-	p.WriteMapBegin(Type(I32), Type(DOUBLE), len(doubleValues))
+	p.WriteMapBegin(types.Type(types.I32), types.Type(types.DOUBLE), len(doubleValues))
 	for k, value := range doubleValues {
 		if e := p.WriteI32(int32(k)); e != nil {
 			t.Fatalf("Unable to write %s key int32 value %v due to error: %s", thetype, k, e.Error())
@@ -660,11 +662,11 @@ func TestWriteJSONProtocolMap(t *testing.T) {
 	if err != nil {
 		t.Fatalf("Error while reading map begin: %s", err.Error())
 	}
-	if expectedKeyType != I32 {
-		t.Fatal("Expected map key type ", I32, ", but was ", expectedKeyType)
+	if expectedKeyType != types.I32 {
+		t.Fatal("Expected map key type ", types.I32, ", but was ", expectedKeyType)
 	}
-	if expectedValueType != DOUBLE {
-		t.Fatal("Expected map value type ", DOUBLE, ", but was ", expectedValueType)
+	if expectedValueType != types.DOUBLE {
+		t.Fatal("Expected map value type ", types.DOUBLE, ", but was ", expectedValueType)
 	}
 	if expectedSize != len(doubleValues) {
 		t.Fatal("Expected map size of ", len(doubleValues), ", but was ", expectedSize)
@@ -684,15 +686,15 @@ func TestWriteJSONProtocolMap(t *testing.T) {
 		s := strconv.FormatFloat(dv, 'g', 10, 64)
 		if math.IsInf(value, 1) {
 			if !math.IsInf(dv, 1) {
-				t.Fatalf("Bad value for %s at index %v %v, wrote: %v, expected: %v", thetype, k, value, s, jsonQuote(JSON_INFINITY))
+				t.Fatalf("Bad value for %s at index %v %v, wrote: %v, expected: %v", thetype, k, value, s, jsonQuote(types.JSON_INFINITY))
 			}
 		} else if math.IsInf(value, 0) {
 			if !math.IsInf(dv, 0) {
-				t.Fatalf("Bad value for %s at index %v %v, wrote: %v, expected: %v", thetype, k, value, s, jsonQuote(JSON_NEGATIVE_INFINITY))
+				t.Fatalf("Bad value for %s at index %v %v, wrote: %v, expected: %v", thetype, k, value, s, jsonQuote(types.JSON_NEGATIVE_INFINITY))
 			}
 		} else if math.IsNaN(value) {
 			if !math.IsNaN(dv) {
-				t.Fatalf("Bad value for %s at index %v  %v, wrote: %v, expected: %v", thetype, k, value, s, jsonQuote(JSON_NAN))
+				t.Fatalf("Bad value for %s at index %v  %v, wrote: %v, expected: %v", thetype, k, value, s, jsonQuote(types.JSON_NAN))
 			}
 		} else {
 			expected := strconv.FormatFloat(value, 'g', 10, 64)
