@@ -19,19 +19,21 @@ package thrift
 import (
 	"fmt"
 	"net"
+
+	"github.com/facebook/fbthrift/thrift/lib/go/thrift/types"
 )
 
 // NewSimpleServer creates a new server that only supports Header Transport.
-func NewSimpleServer(processor Processor, listener net.Listener, transportType TransportID, options ...func(*ServerOptions)) Server {
+func NewSimpleServer(processor types.Processor, listener net.Listener, transportType TransportID, options ...func(*ServerOptions)) Server {
 	serverOptions := simpleServerOptions(options...)
 	processor = WrapInterceptor(serverOptions.interceptor, processor)
 	switch transportType {
 	case TransportIDHeader:
-		return newHeaderServer(processor, listener)
+		return newHeaderServer(processor, listener, serverOptions)
 	case TransportIDRocket:
-		return newRocketServer(processor, listener)
+		return newRocketServer(processor, listener, serverOptions)
 	case TransportIDUpgradeToRocket:
-		return newUpgradeToRocketServer(processor, listener)
+		return newUpgradeToRocketServer(processor, listener, serverOptions)
 	default:
 		panic(fmt.Sprintf("SimpleServer does not support: %v", transportType))
 	}
