@@ -33,6 +33,7 @@ import com.facebook.thrift.util.RpcClientUtils;
 import io.netty.buffer.ByteBuf;
 import io.netty.buffer.ByteBufAllocator;
 import io.netty.buffer.Unpooled;
+import io.netty.util.ReferenceCountUtil;
 import io.rsocket.Payload;
 import io.rsocket.RSocket;
 import io.rsocket.RSocketErrorException;
@@ -51,11 +52,14 @@ import org.apache.thrift.TException;
 import org.apache.thrift.protocol.TStruct;
 import org.apache.thrift.transport.TTransportException;
 import org.reactivestreams.Publisher;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import reactor.core.Exceptions;
 import reactor.core.publisher.Flux;
 import reactor.core.publisher.Mono;
 
 public final class RSocketRpcClient implements RpcClient {
+  private static final Logger LOG = LoggerFactory.getLogger(RSocketRpcClient.class);
   private static final TStruct PARAMETERS_STRUCT = new TStruct();
 
   private final RSocket rsocket;
@@ -72,10 +76,15 @@ public final class RSocketRpcClient implements RpcClient {
   }
 
   @Override
-  public void close() {
+  public void dispose() {
     rsocket.dispose();
   }
 
+  @Override
+  public boolean isDisposed() {
+    return rsocket.isDisposed();
+  }
+
   private boolean hasTimeoutSettings(RpcOptions options) {
     if (options == null) {
       return false;
@@ -100,26 +109,25 @@ public final class RSocketRpcClient implements RpcClient {
   @Override
   public <T> Mono<ClientResponsePayload<T>> singleRequestSingleResponse(
       ClientRequestPayload<T> payload, RpcOptions options) {
+    Payload rsocketPayload = null;
     try {
       final ProtocolId protocol = payload.getRequestRpcMetadata().getProtocol();
       final TProtocolType protocolType = TProtocolType.fromProtocolId(protocol);
 
+      rsocketPayload = clientRequestPayloadToRSocketPayload(payload, protocolType, options);
       return rsocket
-          .requestResponse(clientRequestPayloadToRSocketPayload(payload, protocolType, options))
+          .requestResponse(rsocketPayload)
           .map(response -> rsocketPayloadToClientResponsePayload(payload, response, protocolType))
           .onErrorResume(
               t -> {
                 if (isInternalError(t)) {
                   return Mono.just(getErrorFrame(t));
                 }
-                return Mono.just(
-                    rsocketPayloadToClientResponsePayload(
-                        payload,
-                        ByteBufPayload.create(
-                            getExceptionString(t, payload.getRequestRpcMetadata().getName())),
-                        protocolType));
+                return Mono.error(t);
               });
     } catch (Throwable t) {
+      ReferenceCountUtil.safeRelease(rsocketPayload);
+
       return Mono.error(t);
     }
   }
@@ -150,8 +158,8 @@ public final class RSocketRpcClient implements RpcClient {
    * rocket protocol. Payload will be compact protocol serialized ResponseRpcError struct.
    *
    * @param t Throwable contains rsocket error code
-   * @return ClientResponsePayload
    * @param <T>
+   * @return ClientResponsePayload
    */
   private <T> ClientResponsePayload<T> getErrorFrame(Throwable t) {
     byte[] bytes = t.getMessage().getBytes();
@@ -165,13 +173,16 @@ public final class RSocketRpcClient implements RpcClient {
   @Override
   public <T> Mono<Void> singleRequestNoResponse(
       ClientRequestPayload<T> payload, RpcOptions options) {
+    Payload rsocketPayload = null;
     try {
       final ProtocolId protocol = payload.getRequestRpcMetadata().getProtocol();
       final TProtocolType protocolType = TProtocolType.fromProtocolId(protocol);
 
-      return rsocket.fireAndForget(
-          clientRequestPayloadToRSocketPayload(payload, protocolType, options));
+      rsocketPayload = clientRequestPayloadToRSocketPayload(payload, protocolType, options);
+      return rsocket.fireAndForget(rsocketPayload);
     } catch (Throwable t) {
+      ReferenceCountUtil.safeRelease(rsocketPayload);
+
       return Mono.error(t);
     }
   }
@@ -179,19 +190,27 @@ public final class RSocketRpcClient implements RpcClient {
   @Override
   public <T, K> Flux<ClientResponsePayload<K>> singleRequestStreamingResponse(
       ClientRequestPayload<T> payload, RpcOptions options) {
+    Payload rsocketPayload = null;
     try {
       final ProtocolId protocol = payload.getRequestRpcMetadata().getProtocol();
       final TProtocolType protocolType = TProtocolType.fromProtocolId(protocol);
 
+      rsocketPayload = clientRequestPayloadToRSocketPayload(payload, protocolType, options);
       return rsocket
-          .requestStream(clientRequestPayloadToRSocketPayload(payload, protocolType, options))
+          .requestStream(rsocketPayload)
           .onErrorResume(
-              t ->
-                  Flux.just(
+              t -> {
+                if (isInternalError(t)) {
+                  return Flux.just(
                       ByteBufPayload.create(
-                          getExceptionString(t, payload.getRequestRpcMetadata().getName()))))
+                          getExceptionString(t, payload.getRequestRpcMetadata().getName())));
+                }
+                return Flux.error(t);
+              })
           .map(new StreamingResponseHandler<>(payload));
     } catch (Throwable t) {
+      ReferenceCountUtil.safeRelease(rsocketPayload);
+
       return Flux.error(t);
     }
   }
@@ -216,12 +235,16 @@ public final class RSocketRpcClient implements RpcClient {
 
                   return requestChannel
                       .onErrorResume(
-                          t ->
-                              Flux.just(
+                          t -> {
+                            if (isInternalError(t)) {
+                              return Flux.just(
                                   ByteBufPayload.create(
                                       getExceptionString(
                                           t,
-                                          clientRequestPayload.getRequestRpcMetadata().getName()))))
+                                          clientRequestPayload.getRequestRpcMetadata().getName())));
+                            }
+                            return Flux.error(t);
+                          })
                       .map(new StreamingResponseHandler<>(clientRequestPayload));
                 });
 
@@ -239,6 +262,7 @@ public final class RSocketRpcClient implements RpcClient {
       Payload payload = ByteBufPayload.create(alloc.buffer(), metadata);
       return rsocket.metadataPush(payload);
     } catch (Throwable t) {
+      ReferenceCountUtil.safeRelease(metadata);
       return Mono.error(t);
     }
   }
