@@ -27,6 +27,8 @@ from typing import Optional, Sequence
 from derived.thrift_clients import DerivedTestingService
 from derived.thrift_services import DerivedTestingServiceInterface
 from folly.iobuf import IOBuf
+
+from parameterized import parameterized_class
 from stack_args.thrift_clients import StackService
 from stack_args.thrift_services import StackServiceInterface
 from stack_args.thrift_types import simple
@@ -37,6 +39,9 @@ from thrift.py3.server import get_context, SocketAddress
 from thrift.python.client import get_client
 from thrift.python.common import Priority, RpcOptions
 from thrift.python.exceptions import ApplicationError
+from thrift.python.flagged.test.enable_resource_pools_for_python_test import (
+    mock_enable_resource_pools_for_python,
+)
 from thrift.python.server import ServiceInterface, ThriftServer
 
 
@@ -103,7 +108,6 @@ class DerivedHandler(Handler, DerivedTestingServiceInterface):
         return color
 
 
-# pyre-fixme[13]: Attribute `serve_task` is never initialized.
 class TestServer:
     server: ThriftServer
     serve_task: asyncio.Task
@@ -115,6 +119,11 @@ class TestServer:
         handler: ServiceInterface = Handler(),  # noqa: B008
     ) -> None:
         self.server = ThriftServer(handler, ip=ip, path=path)
+        # pyre-fixme[4, 8]: The initialization below eliminates
+        #                   the pyre[13] error, but results in
+        #                   pyre[4] and pyre[8] errors.
+        #                   __aenter__ sets the required value.
+        self.serve_task = None
 
     async def __aenter__(self) -> SocketAddress:
         self.serve_task = asyncio.get_event_loop().create_task(self.server.serve())
@@ -126,11 +135,19 @@ class TestServer:
         await self.serve_task
 
 
+@parameterized_class(
+    ("enable_resource_pools_for_python"),
+    [(True,), (False,)],
+)
 class ClientServerTests(unittest.IsolatedAsyncioTestCase):
     """
     These are tests where a client and server talk to each other
     """
 
+    def setUp(self) -> None:
+        # pyre-fixme[16]: `ClientServerTests` has no attribute `enable_resource_pools_for_python`
+        mock_enable_resource_pools_for_python(self.enable_resource_pools_for_python)
+
     async def test_get_context(self) -> None:
 
         async with TestServer(ip="::1") as sa:
@@ -390,11 +407,19 @@ class StackHandler(StackServiceInterface):
             raise Exception("WRONG")
 
 
+@parameterized_class(
+    ("enable_resource_pools_for_python"),
+    [(True,), (False,)],
+)
 class ClientStackServerTests(unittest.IsolatedAsyncioTestCase):
     """
     These are tests where a client and server(stack_arguments) talk to each other
     """
 
+    def setUp(self) -> None:
+        # pyre-fixme[16]: `ClientServerTests` has no attribute `enable_resource_pools_for_python`
+        mock_enable_resource_pools_for_python(self.enable_resource_pools_for_python)
+
     async def test_server_localhost(self) -> None:
 
         async with TestServer(handler=StackHandler(), ip="::1") as sa:
