@@ -175,6 +175,13 @@ std::tuple<UniquePyObjectPtr, bool> getStandardImmutableDefaultValueForType(
 std::tuple<UniquePyObjectPtr, bool> getStandardMutableDefaultValueForType(
     const detail::TypeInfo& typeInfo) {
   UniquePyObjectPtr value = nullptr;
+  // Immutable types use a default-value cache and initialize the fields with
+  // the same cached Python object repeatedly. However, this approach does not
+  // work well with mutable types when there is no copy-on-write mechanism in
+  // place. The issue is particularly problematic with container types and
+  // structs/unions. However, since we deep copy the default values for mutable
+  // types above the cache layer, it is fine to keep `addValueToCache = true`
+  // even for containers.
   bool addValueToCache = true;
   switch (typeInfo.type) {
     case protocol::TType::T_BYTE:
@@ -231,7 +238,7 @@ std::tuple<UniquePyObjectPtr, bool> getStandardMutableDefaultValueForType(
           static_cast<const detail::StructInfo*>(typeInfo.typeExt);
       value = UniquePyObjectPtr(
           structInfo->unionExt != nullptr
-              ? createUnionTuple()
+              ? createMutableUnionDataHolder()
               : createStructListWithDefaultValues(
                     *structInfo, getStandardMutableDefaultValueForType));
       break;
@@ -332,6 +339,13 @@ struct TupleContainer final {
   static bool Check(const PyObject* p) { return PyTuple_Check(p); }
 
   static Py_ssize_t Size(PyObject* p) { return PyTuple_Size(p); }
+
+  /**
+   * Returns a deep copy of the given object. Since TupleContainer acts as a
+   * data holder policy for immutable types, this operation simply returns the
+   * original object, as no actual copying is necessary for immutable data.
+   */
+  static PyObject* deepCopy(PyObject* p) { return p; }
 };
 
 /*
@@ -374,6 +388,16 @@ struct ListContainer final {
   static bool Check(const PyObject* p) { return PyList_Check(p); }
 
   static Py_ssize_t Size(PyObject* p) { return PyList_Size(p); }
+
+  /**
+   * Returns a deep copy of the given object. Since ListContainer acts as a
+   * data holder policy for mutable types, a real deep copy is necessary. For
+   * immutable types, the deep copy operation simply returns the object itself.
+   */
+  static PyObject* deepCopy(PyObject* p) {
+    ensureImportOrThrow();
+    return deepcopy(p);
+  }
 };
 
 /**
@@ -441,7 +465,10 @@ PyObject* createStructContainerWithDefaultValues(
               .release());
     }
   }
-  return container.release();
+
+  // The policy determines the actual deep copy operation; it performs a no-op
+  // for immutable types.
+  return Container::deepCopy(container.release());
 }
 
 /**
@@ -513,12 +540,12 @@ void populateStructContainerUnsetFieldsWithDefaultValues(
       Container::SET_ITEM(
           container,
           i + 1,
-          getDefaultValueForField(
-              fieldInfo.typeInfo,
-              defaultValues,
-              i,
-              getStandardDefaultValueForTypeFunc)
-              .release());
+          Container::deepCopy(getDefaultValueForField(
+                                  fieldInfo.typeInfo,
+                                  defaultValues,
+                                  i,
+                                  getStandardDefaultValueForTypeFunc)
+                                  .release()));
     }
     Py_DECREF(oldValue);
   }
@@ -564,7 +591,8 @@ void* setMutableStruct(void* objectPtr, const detail::TypeInfo& typeInfo) {
   return getListObjectItemBase(list);
 }
 
-void* setUnion(void* objectPtr, const detail::TypeInfo& /* typeInfo */) {
+void* setImmutableUnion(
+    void* objectPtr, const detail::TypeInfo& /* typeInfo */) {
   return setPyObject(objectPtr, UniquePyObjectPtr{createUnionTuple()});
 }
 
@@ -578,6 +606,13 @@ bool getImmutableIsset(const void* object, ptrdiff_t offset) {
   return flags[offset];
 }
 
+void* setMutableUnion(void* objectPtr, const detail::TypeInfo& /* typeInfo */) {
+  PyObject* list =
+      setPyObject(objectPtr, UniquePyObjectPtr{createMutableUnionDataHolder()});
+
+  return getListObjectItemBase(list);
+}
+
 /**
  * Gets the "isset" flag of the `index`-th field of the 'struct list'
  *
@@ -643,6 +678,25 @@ void clearImmutableUnion(void* object) {
   clearUnionDataHolder<TupleContainer>(object);
 }
 
+void clearMutableUnion(void* object) {
+  PyObject** unionDataHolder = toPyObjectPtr(object);
+
+  // Clear field id of "present field" (if any).
+  PyObject* const previousActiveFieldId = *unionDataHolder;
+  UniquePyObjectPtr zero{PyLong_FromLong(0)};
+  if (zero == nullptr) {
+    THRIFT_PY3_CHECK_ERROR();
+  }
+  *unionDataHolder = zero.release();
+  Py_XDECREF(previousActiveFieldId);
+
+  // Clear value (if any).
+  PyObject* const previousValue = *++unionDataHolder;
+  *unionDataHolder = Py_None;
+  Py_INCREF(Py_None);
+  Py_XDECREF(previousValue);
+}
+
 /**
  * Returns the id of the field that is currently set for the given union data
  * holder, or 0 if the union is empty.
@@ -665,6 +719,15 @@ int getImmutableUnionActiveFieldId(const void* object) {
   return getUnionDataHolderActiveFieldId<TupleContainer>(object);
 }
 
+int getMutableUnionActiveFieldId(const void* object) {
+  PyObject* const unionDataHolder = *toPyObjectPtr(object);
+  const long id = PyLong_AsLong(unionDataHolder);
+  if (id == -1) {
+    THRIFT_PY3_CHECK_POSSIBLE_ERROR();
+  }
+  return id;
+}
+
 /**
  * Updates the given union data holder to indicate that the given fieldId is
  * currently set for that union.
@@ -691,6 +754,18 @@ void setImmutableUnionActiveFieldId(void* object, int fieldId) {
   setUnionDataHolderActiveFieldId<TupleContainer>(object, fieldId);
 }
 
+void setMutableUnionActiveFieldId(void* object, int fieldId) {
+  UniquePyObjectPtr fieldIdPyObj{PyLong_FromLong(fieldId)};
+  if (fieldIdPyObj == nullptr) {
+    THRIFT_PY3_CHECK_ERROR();
+  }
+
+  PyObject** unionDataHolder = toPyObjectPtr(object);
+  PyObject* previousFieldId = *unionDataHolder;
+  *unionDataHolder = fieldIdPyObj.release();
+  Py_DECREF(previousFieldId);
+}
+
 const detail::UnionExtN<1> kImmutableUnionExt = {
     /* .clear */ clearImmutableUnion,
     /* .unionTypeOffset */ 0,
@@ -699,6 +774,24 @@ const detail::UnionExtN<1> kImmutableUnionExt = {
     /* .initMember */ {nullptr},
 };
 
+const detail::UnionExtN<1> kMutableUnionExt = {
+    /* .clear */ clearMutableUnion,
+    /* .unionTypeOffset */ 0,
+    /* .getActiveId */ getMutableUnionActiveFieldId,
+    /* .setActiveId */ setMutableUnionActiveFieldId,
+    /* .initMember */ {nullptr},
+};
+
+const detail::UnionExt* FOLLY_NULLABLE
+getStructInfoUnionExt(bool isUnion, bool isMutable) {
+  if (!isUnion) {
+    return nullptr;
+  }
+
+  return reinterpret_cast<const detail::UnionExt*>(
+      isMutable ? &kMutableUnionExt : &kImmutableUnionExt);
+}
+
 /**
  * Creates a new (table-based) serializer StructInfo for the thrift-python
  * structured type (struct, union or exception) with the given properties.
@@ -729,9 +822,7 @@ detail::StructInfo* newTableBasedSerializerStructInfo(
       std::align_val_t{alignof(detail::StructInfo)}));
   structInfo->numFields = numFields;
   structInfo->name = namePtr;
-  structInfo->unionExt = isUnion
-      ? reinterpret_cast<const detail::UnionExt*>(&kImmutableUnionExt)
-      : nullptr;
+  structInfo->unionExt = getStructInfoUnionExt(isUnion, isMutable);
   structInfo->getIsset = isMutable ? getMutableIsset : getImmutableIsset;
   structInfo->setIsset = isMutable ? setMutableIsset : setIsset;
   structInfo->getFieldValuesBasePtr = nullptr;
@@ -1050,10 +1141,16 @@ PyObject* createUnionDataHolder() {
 
 } // namespace
 
+// DO_BEFORE(aristidis,20240920): For consistency, rename method to
+// createImmutableUnionDataHolder() - and update references.
 PyObject* createUnionTuple() {
   return createUnionDataHolder<TupleContainer>();
 }
 
+PyObject* createMutableUnionDataHolder() {
+  return createUnionDataHolder<ListContainer>();
+}
+
 template <typename Container>
 PyObject* createStructContainer(int16_t numFields) {
   // Allocate and 0-initialize numFields bytes.
@@ -1155,6 +1252,7 @@ void populateMutableStructListUnsetFieldsWithDefaultValues(
 
 void resetFieldToStandardDefault(
     PyObject* structList, const detail::StructInfo& structInfo, int index) {
+  ensureImportOrThrow();
   if (structList == nullptr) {
     throw std::runtime_error(fmt::format(
         "Received null list while resetting struct:`{}`, field-index:'{}'",
@@ -1172,22 +1270,22 @@ void resetFieldToStandardDefault(
   if (fieldInfo.qualifier == detail::FieldQualifier::Optional) {
     PyList_SET_ITEM(structList, index + 1, Py_None);
     Py_INCREF(Py_None);
+    setMutableStructIsset(structList, index, false);
   } else {
     // getDefaultValueForField calls `Py_INCREF`
+    // This function is called only to reset the fields of mutable types.
+    // Therefore, a deep copy of the given default field value is necessary.
     PyList_SET_ITEM(
         structList,
         index + 1,
-        getDefaultValueForField(
-            fieldInfo.typeInfo,
-            defaultValues,
-            index,
-            getStandardMutableDefaultValueForType)
-            .release());
+        deepcopy(getDefaultValueForField(
+                     fieldInfo.typeInfo,
+                     defaultValues,
+                     index,
+                     getStandardMutableDefaultValueForType)
+                     .release()));
   }
   Py_DECREF(oldValue);
-  // DO_BEFORE(alperyoney,20240515): Figure out whether isset flag should be
-  // cleared for non-optional fields.
-  setMutableStructIsset(structList, index, false);
 }
 
 detail::OptionalThriftValue getStruct(
@@ -1203,7 +1301,7 @@ detail::TypeInfo createImmutableStructTypeInfo(
       /* .get */ getStruct,
       /* .set */
       reinterpret_cast<detail::VoidFuncPtr>(
-          dynamicStructInfo.isUnion() ? setUnion : setImmutableStruct),
+          dynamicStructInfo.isUnion() ? setImmutableUnion : setImmutableStruct),
       /* .typeExt */ &dynamicStructInfo.getStructInfo(),
   };
 }
@@ -1221,7 +1319,7 @@ detail::TypeInfo createMutableStructTypeInfo(
       /* .get */ getMutableStruct,
       /* .set */
       reinterpret_cast<detail::VoidFuncPtr>(
-          dynamicStructInfo.isUnion() ? setUnion : setMutableStruct),
+          dynamicStructInfo.isUnion() ? setMutableUnion : setMutableStruct),
       /* .typeExt */ &dynamicStructInfo.getStructInfo(),
   };
 }
