@@ -22,8 +22,8 @@
 
 #include <folly/Conv.h>
 #include <folly/Format.h>
-#include <folly/experimental/TestUtil.h>
 #include <folly/portability/Unistd.h>
+#include <folly/testing/TestUtil.h>
 
 #include <thrift/lib/cpp/transport/TBufferTransports.h>
 #include <thrift/lib/cpp/transport/TFDTransport.h>
